import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Optional, Type, TypeVar, Union, cast

import attr
from dateutil.parser import isoparse

from ..models.organization_addon_billing_period import OrganizationAddonBillingPeriod
from ..models.organization_flags_item import OrganizationFlagsItem
from ..models.organization_invoice_billing_period import OrganizationInvoiceBillingPeriod
from ..models.organization_plan import OrganizationPlan
from ..models.organization_support_tier import OrganizationSupportTier
from ..models.organization_supported_regions_item import OrganizationSupportedRegionsItem
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.aws_info import AwsInfo
    from ..models.billing_info import BillingInfo
    from ..models.device_configuration_document import DeviceConfigurationDocument
    from ..models.google_info import GoogleInfo
    from ..models.google_storage_info import GoogleStorageInfo
    from ..models.looker_info import LookerInfo
    from ..models.overview_settings import OverviewSettings
    from ..models.pagerduty_info import PagerdutyInfo
    from ..models.rtc_info import RtcInfo
    from ..models.slack_info import SlackInfo
    from ..models.stripe_info import StripeInfo
    from ..models.teleop_high_ping_reconnect_behaviors import TeleopHighPingReconnectBehaviors
    from ..models.user_teleop_configuration import UserTeleopConfiguration
    from ..models.webhooks_info import WebhooksInfo


T = TypeVar("T", bound="Organization")


@attr.s(auto_attribs=True)
class Organization:
    """
    Attributes:
        plan (OrganizationPlan):
        name (str):
        industry (str):
        website (str):
        address_line_1 (str):
        address_line_2 (str):
        city (str):
        state (str):
        postal_code (str):
        country (str):
        addon_billing_period (OrganizationAddonBillingPeriod):
        invoice_billing_period (OrganizationInvoiceBillingPeriod):
        description (Union[Unset, None, str]):
        enabled (Union[Unset, bool]):
        pagerduty_info (Union[Unset, PagerdutyInfo]):
        teleop_high_ping_reconnect_behaviors (Union[Unset, TeleopHighPingReconnectBehaviors]):
        slack_info (Union[Unset, SlackInfo]):
        google_info (Union[Unset, GoogleInfo]):
        webhooks_info (Union[Unset, WebhooksInfo]):
        aws_info (Union[Unset, AwsInfo]):
        google_storage_info (Union[Unset, GoogleStorageInfo]):
        looker_info (Union[Unset, LookerInfo]):
        stripe_info (Union[Unset, StripeInfo]):
        rtc_info (Union[Unset, RtcInfo]):
        teleop_configuration (Union[Unset, UserTeleopConfiguration]):
        analytics_enabled (Union[Unset, None, datetime.datetime]):
        data_export_enabled (Union[Unset, None, datetime.datetime]):
        advanced_configuration_enabled (Union[Unset, None, datetime.datetime]):
        customer_portal_enabled (Union[Unset, None, datetime.datetime]):
        stripe_billing_enabled (Union[Unset, bool]):
        stripe_subscription_enabled (Union[Unset, bool]):
        mission_planning_enabled (Union[Unset, bool]):
        billing_info (Union[Unset, BillingInfo]):
        s_3_export_enabled (Union[Unset, bool]):
        blob_data_enabled (Union[Unset, bool]):
        white_label_enabled (Union[Unset, bool]):
        viewer_3_d_enabled (Union[Unset, bool]):
        adapters_enabled (Union[Unset, bool]):
        white_label_css (Union[Unset, None, str]):
        demo_mode_enabled (Union[Unset, bool]):
        teleop_share_enabled (Union[Unset, bool]):
        bill_estimate_enabled (Union[Unset, bool]):
        data_retention_enabled (Union[Unset, None, datetime.datetime]):
        days_data_retained (Union[Unset, int]):
        max_chunk_request_limit (Union[Unset, int]):
        online_sensitivity (Union[Unset, int]):
        support_enabled (Union[Unset, None, datetime.datetime]):
        support_tier (Union[Unset, OrganizationSupportTier]):
        trial_period_end (Optional[datetime.datetime]):
        external_id (Union[Unset, str]):
        chargebee_id (Optional[str]):
        external_twilio_config (Optional[str]):
        totango_id (Optional[str]):
        hubspot_id (Optional[str]):
        custom_tos (Union[Unset, bool]):
        teleop_enabled (Union[Unset, None, datetime.datetime]):
        observability_enabled (Union[Unset, None, datetime.datetime]):
        share_enabled (Union[Unset, None, datetime.datetime]):
        annotations_enabled (Union[Unset, None, datetime.datetime]):
        diagnostics_enabled (Union[Unset, None, datetime.datetime]):
        ssh_enabled (Union[Unset, None, datetime.datetime]):
        spot_enabled (Union[Unset, None, datetime.datetime]):
        file_storage_enabled (Union[Unset, bool]):
        role_viewer_enabled (Union[Unset, bool]):
        teams_enabled (Union[Unset, bool]):
        schedules_enabled (Union[Unset, bool]):
        beta_ui (Union[Unset, bool]):
        paging_enabled (Union[Unset, bool]):
        stateful_events_enabled (Union[Unset, bool]):
        billing_enabled (Union[Unset, bool]):
        sms_followers_enabled (Union[Unset, bool]):
        event_action_interval_enabled (Union[Unset, bool]):
        task_summaries_enabled (Union[Unset, bool]):
        default_device_config (Union[Unset, DeviceConfigurationDocument]):
        default_stream_throttle_hz (Union[Unset, None, float]):
        outage_banner_enabled (Union[Unset, bool]):
        new_header_enabled (Union[Unset, bool]):
        explorations_enabled (Union[Unset, bool]):
        customer_success_ai_enabled (Union[Unset, bool]):
        custom_analytics_enabled (Union[Unset, bool]):
        fleet_analytics_enabled (Union[Unset, bool]):
        navigator_enabled (Union[Unset, bool]):
        list_view_enabled (Union[Unset, bool]):
        map_view_enabled (Union[Unset, bool]):
        node_graph_page_enabled (Union[Unset, bool]):
        flags (Union[Unset, List[OrganizationFlagsItem]]):
        overview_settings (Union[Unset, OverviewSettings]):
        snowflake_warehouse_name (Optional[str]):
        supported_regions (Union[Unset, List[OrganizationSupportedRegionsItem]]):
        enable_sso_oidc (Union[Unset, bool]):
        inactivity_logout_enabled (Union[Unset, bool]):
        sms_opt_in_enabled (Union[Unset, bool]):
        inactivity_timeout (Union[Unset, float]):
        custom_event_triggers_enabled (Union[Unset, bool]):
        allow_custom_email_configuration (Union[Unset, bool]):
        portal_base_url (Union[Unset, None, str]):
        ingestion_enabled (Union[Unset, bool]):
        timescale_enabled (Optional[datetime.datetime]):
        clickhouse_enabled (Optional[datetime.datetime]):
        clickhouse_query_enabled (Union[Unset, bool]):
        drop_experimental_query_data (Union[Unset, bool]):
        event_queries_enabled (Union[Unset, bool]):
        snowflake_row_level_security_enabled (Union[Unset, bool]):
        clickhouse_analytics_enabled (Union[Unset, bool]):
        cached_events_enabled (Union[Unset, bool]):
        show_get_started_on_overview (Union[Unset, bool]):
        sidebar_hidden_sections (Union[Unset, List[str]]):
        design_fleet_manager_white_listed_users (Union[Unset, None, str]):
        design_fleet_manager_view_name (Union[Unset, None, str]):
        optional_help_link (Union[Unset, None, str]):
        show_overview_filters (Union[Unset, bool]):
        local_mode_enabled (Union[Unset, bool]):
        embed_view_enabled (Union[Unset, bool]):
        id (Union[Unset, str]):
        created_at (Union[Unset, datetime.datetime]):
        updated_at (Union[Unset, datetime.datetime]):
    """

    plan: OrganizationPlan
    name: str
    industry: str
    website: str
    address_line_1: str
    address_line_2: str
    city: str
    state: str
    postal_code: str
    country: str
    addon_billing_period: OrganizationAddonBillingPeriod
    invoice_billing_period: OrganizationInvoiceBillingPeriod
    trial_period_end: Optional[datetime.datetime]
    chargebee_id: Optional[str]
    external_twilio_config: Optional[str]
    totango_id: Optional[str]
    hubspot_id: Optional[str]
    snowflake_warehouse_name: Optional[str]
    timescale_enabled: Optional[datetime.datetime]
    clickhouse_enabled: Optional[datetime.datetime]
    description: Union[Unset, None, str] = UNSET
    enabled: Union[Unset, bool] = UNSET
    pagerduty_info: Union[Unset, "PagerdutyInfo"] = UNSET
    teleop_high_ping_reconnect_behaviors: Union[Unset, "TeleopHighPingReconnectBehaviors"] = UNSET
    slack_info: Union[Unset, "SlackInfo"] = UNSET
    google_info: Union[Unset, "GoogleInfo"] = UNSET
    webhooks_info: Union[Unset, "WebhooksInfo"] = UNSET
    aws_info: Union[Unset, "AwsInfo"] = UNSET
    google_storage_info: Union[Unset, "GoogleStorageInfo"] = UNSET
    looker_info: Union[Unset, "LookerInfo"] = UNSET
    stripe_info: Union[Unset, "StripeInfo"] = UNSET
    rtc_info: Union[Unset, "RtcInfo"] = UNSET
    teleop_configuration: Union[Unset, "UserTeleopConfiguration"] = UNSET
    analytics_enabled: Union[Unset, None, datetime.datetime] = UNSET
    data_export_enabled: Union[Unset, None, datetime.datetime] = UNSET
    advanced_configuration_enabled: Union[Unset, None, datetime.datetime] = UNSET
    customer_portal_enabled: Union[Unset, None, datetime.datetime] = UNSET
    stripe_billing_enabled: Union[Unset, bool] = UNSET
    stripe_subscription_enabled: Union[Unset, bool] = UNSET
    mission_planning_enabled: Union[Unset, bool] = UNSET
    billing_info: Union[Unset, "BillingInfo"] = UNSET
    s_3_export_enabled: Union[Unset, bool] = UNSET
    blob_data_enabled: Union[Unset, bool] = UNSET
    white_label_enabled: Union[Unset, bool] = UNSET
    viewer_3_d_enabled: Union[Unset, bool] = UNSET
    adapters_enabled: Union[Unset, bool] = UNSET
    white_label_css: Union[Unset, None, str] = UNSET
    demo_mode_enabled: Union[Unset, bool] = UNSET
    teleop_share_enabled: Union[Unset, bool] = UNSET
    bill_estimate_enabled: Union[Unset, bool] = UNSET
    data_retention_enabled: Union[Unset, None, datetime.datetime] = UNSET
    days_data_retained: Union[Unset, int] = UNSET
    max_chunk_request_limit: Union[Unset, int] = UNSET
    online_sensitivity: Union[Unset, int] = UNSET
    support_enabled: Union[Unset, None, datetime.datetime] = UNSET
    support_tier: Union[Unset, OrganizationSupportTier] = UNSET
    external_id: Union[Unset, str] = UNSET
    custom_tos: Union[Unset, bool] = UNSET
    teleop_enabled: Union[Unset, None, datetime.datetime] = UNSET
    observability_enabled: Union[Unset, None, datetime.datetime] = UNSET
    share_enabled: Union[Unset, None, datetime.datetime] = UNSET
    annotations_enabled: Union[Unset, None, datetime.datetime] = UNSET
    diagnostics_enabled: Union[Unset, None, datetime.datetime] = UNSET
    ssh_enabled: Union[Unset, None, datetime.datetime] = UNSET
    spot_enabled: Union[Unset, None, datetime.datetime] = UNSET
    file_storage_enabled: Union[Unset, bool] = UNSET
    role_viewer_enabled: Union[Unset, bool] = UNSET
    teams_enabled: Union[Unset, bool] = UNSET
    schedules_enabled: Union[Unset, bool] = UNSET
    beta_ui: Union[Unset, bool] = UNSET
    paging_enabled: Union[Unset, bool] = UNSET
    stateful_events_enabled: Union[Unset, bool] = UNSET
    billing_enabled: Union[Unset, bool] = UNSET
    sms_followers_enabled: Union[Unset, bool] = UNSET
    event_action_interval_enabled: Union[Unset, bool] = UNSET
    task_summaries_enabled: Union[Unset, bool] = UNSET
    default_device_config: Union[Unset, "DeviceConfigurationDocument"] = UNSET
    default_stream_throttle_hz: Union[Unset, None, float] = UNSET
    outage_banner_enabled: Union[Unset, bool] = UNSET
    new_header_enabled: Union[Unset, bool] = UNSET
    explorations_enabled: Union[Unset, bool] = UNSET
    customer_success_ai_enabled: Union[Unset, bool] = UNSET
    custom_analytics_enabled: Union[Unset, bool] = UNSET
    fleet_analytics_enabled: Union[Unset, bool] = UNSET
    navigator_enabled: Union[Unset, bool] = UNSET
    list_view_enabled: Union[Unset, bool] = UNSET
    map_view_enabled: Union[Unset, bool] = UNSET
    node_graph_page_enabled: Union[Unset, bool] = UNSET
    flags: Union[Unset, List[OrganizationFlagsItem]] = UNSET
    overview_settings: Union[Unset, "OverviewSettings"] = UNSET
    supported_regions: Union[Unset, List[OrganizationSupportedRegionsItem]] = UNSET
    enable_sso_oidc: Union[Unset, bool] = UNSET
    inactivity_logout_enabled: Union[Unset, bool] = UNSET
    sms_opt_in_enabled: Union[Unset, bool] = UNSET
    inactivity_timeout: Union[Unset, float] = UNSET
    custom_event_triggers_enabled: Union[Unset, bool] = UNSET
    allow_custom_email_configuration: Union[Unset, bool] = UNSET
    portal_base_url: Union[Unset, None, str] = UNSET
    ingestion_enabled: Union[Unset, bool] = UNSET
    clickhouse_query_enabled: Union[Unset, bool] = UNSET
    drop_experimental_query_data: Union[Unset, bool] = UNSET
    event_queries_enabled: Union[Unset, bool] = UNSET
    snowflake_row_level_security_enabled: Union[Unset, bool] = UNSET
    clickhouse_analytics_enabled: Union[Unset, bool] = UNSET
    cached_events_enabled: Union[Unset, bool] = UNSET
    show_get_started_on_overview: Union[Unset, bool] = UNSET
    sidebar_hidden_sections: Union[Unset, List[str]] = UNSET
    design_fleet_manager_white_listed_users: Union[Unset, None, str] = UNSET
    design_fleet_manager_view_name: Union[Unset, None, str] = UNSET
    optional_help_link: Union[Unset, None, str] = UNSET
    show_overview_filters: Union[Unset, bool] = UNSET
    local_mode_enabled: Union[Unset, bool] = UNSET
    embed_view_enabled: Union[Unset, bool] = UNSET
    id: Union[Unset, str] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        plan = self.plan.value

        name = self.name
        industry = self.industry
        website = self.website
        address_line_1 = self.address_line_1
        address_line_2 = self.address_line_2
        city = self.city
        state = self.state
        postal_code = self.postal_code
        country = self.country
        addon_billing_period = self.addon_billing_period.value

        invoice_billing_period = self.invoice_billing_period.value

        description = self.description
        enabled = self.enabled
        pagerduty_info: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.pagerduty_info, Unset):
            pagerduty_info = self.pagerduty_info.to_dict()

        teleop_high_ping_reconnect_behaviors: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.teleop_high_ping_reconnect_behaviors, Unset):
            teleop_high_ping_reconnect_behaviors = self.teleop_high_ping_reconnect_behaviors.to_dict()

        slack_info: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.slack_info, Unset):
            slack_info = self.slack_info.to_dict()

        google_info: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.google_info, Unset):
            google_info = self.google_info.to_dict()

        webhooks_info: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.webhooks_info, Unset):
            webhooks_info = self.webhooks_info.to_dict()

        aws_info: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.aws_info, Unset):
            aws_info = self.aws_info.to_dict()

        google_storage_info: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.google_storage_info, Unset):
            google_storage_info = self.google_storage_info.to_dict()

        looker_info: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.looker_info, Unset):
            looker_info = self.looker_info.to_dict()

        stripe_info: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.stripe_info, Unset):
            stripe_info = self.stripe_info.to_dict()

        rtc_info: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.rtc_info, Unset):
            rtc_info = self.rtc_info.to_dict()

        teleop_configuration: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.teleop_configuration, Unset):
            teleop_configuration = self.teleop_configuration.to_dict()

        analytics_enabled: Union[Unset, None, str] = UNSET
        if not isinstance(self.analytics_enabled, Unset):
            analytics_enabled = self.analytics_enabled.isoformat() if self.analytics_enabled else None

        data_export_enabled: Union[Unset, None, str] = UNSET
        if not isinstance(self.data_export_enabled, Unset):
            data_export_enabled = self.data_export_enabled.isoformat() if self.data_export_enabled else None

        advanced_configuration_enabled: Union[Unset, None, str] = UNSET
        if not isinstance(self.advanced_configuration_enabled, Unset):
            advanced_configuration_enabled = (
                self.advanced_configuration_enabled.isoformat() if self.advanced_configuration_enabled else None
            )

        customer_portal_enabled: Union[Unset, None, str] = UNSET
        if not isinstance(self.customer_portal_enabled, Unset):
            customer_portal_enabled = self.customer_portal_enabled.isoformat() if self.customer_portal_enabled else None

        stripe_billing_enabled = self.stripe_billing_enabled
        stripe_subscription_enabled = self.stripe_subscription_enabled
        mission_planning_enabled = self.mission_planning_enabled
        billing_info: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.billing_info, Unset):
            billing_info = self.billing_info.to_dict()

        s_3_export_enabled = self.s_3_export_enabled
        blob_data_enabled = self.blob_data_enabled
        white_label_enabled = self.white_label_enabled
        viewer_3_d_enabled = self.viewer_3_d_enabled
        adapters_enabled = self.adapters_enabled
        white_label_css = self.white_label_css
        demo_mode_enabled = self.demo_mode_enabled
        teleop_share_enabled = self.teleop_share_enabled
        bill_estimate_enabled = self.bill_estimate_enabled
        data_retention_enabled: Union[Unset, None, str] = UNSET
        if not isinstance(self.data_retention_enabled, Unset):
            data_retention_enabled = self.data_retention_enabled.isoformat() if self.data_retention_enabled else None

        days_data_retained = self.days_data_retained
        max_chunk_request_limit = self.max_chunk_request_limit
        online_sensitivity = self.online_sensitivity
        support_enabled: Union[Unset, None, str] = UNSET
        if not isinstance(self.support_enabled, Unset):
            support_enabled = self.support_enabled.isoformat() if self.support_enabled else None

        support_tier: Union[Unset, str] = UNSET
        if not isinstance(self.support_tier, Unset):
            support_tier = self.support_tier.value

        trial_period_end = self.trial_period_end.isoformat() if self.trial_period_end else None

        external_id = self.external_id
        chargebee_id = self.chargebee_id
        external_twilio_config = self.external_twilio_config
        totango_id = self.totango_id
        hubspot_id = self.hubspot_id
        custom_tos = self.custom_tos
        teleop_enabled: Union[Unset, None, str] = UNSET
        if not isinstance(self.teleop_enabled, Unset):
            teleop_enabled = self.teleop_enabled.isoformat() if self.teleop_enabled else None

        observability_enabled: Union[Unset, None, str] = UNSET
        if not isinstance(self.observability_enabled, Unset):
            observability_enabled = self.observability_enabled.isoformat() if self.observability_enabled else None

        share_enabled: Union[Unset, None, str] = UNSET
        if not isinstance(self.share_enabled, Unset):
            share_enabled = self.share_enabled.isoformat() if self.share_enabled else None

        annotations_enabled: Union[Unset, None, str] = UNSET
        if not isinstance(self.annotations_enabled, Unset):
            annotations_enabled = self.annotations_enabled.isoformat() if self.annotations_enabled else None

        diagnostics_enabled: Union[Unset, None, str] = UNSET
        if not isinstance(self.diagnostics_enabled, Unset):
            diagnostics_enabled = self.diagnostics_enabled.isoformat() if self.diagnostics_enabled else None

        ssh_enabled: Union[Unset, None, str] = UNSET
        if not isinstance(self.ssh_enabled, Unset):
            ssh_enabled = self.ssh_enabled.isoformat() if self.ssh_enabled else None

        spot_enabled: Union[Unset, None, str] = UNSET
        if not isinstance(self.spot_enabled, Unset):
            spot_enabled = self.spot_enabled.isoformat() if self.spot_enabled else None

        file_storage_enabled = self.file_storage_enabled
        role_viewer_enabled = self.role_viewer_enabled
        teams_enabled = self.teams_enabled
        schedules_enabled = self.schedules_enabled
        beta_ui = self.beta_ui
        paging_enabled = self.paging_enabled
        stateful_events_enabled = self.stateful_events_enabled
        billing_enabled = self.billing_enabled
        sms_followers_enabled = self.sms_followers_enabled
        event_action_interval_enabled = self.event_action_interval_enabled
        task_summaries_enabled = self.task_summaries_enabled
        default_device_config: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.default_device_config, Unset):
            default_device_config = self.default_device_config.to_dict()

        default_stream_throttle_hz = self.default_stream_throttle_hz
        outage_banner_enabled = self.outage_banner_enabled
        new_header_enabled = self.new_header_enabled
        explorations_enabled = self.explorations_enabled
        customer_success_ai_enabled = self.customer_success_ai_enabled
        custom_analytics_enabled = self.custom_analytics_enabled
        fleet_analytics_enabled = self.fleet_analytics_enabled
        navigator_enabled = self.navigator_enabled
        list_view_enabled = self.list_view_enabled
        map_view_enabled = self.map_view_enabled
        node_graph_page_enabled = self.node_graph_page_enabled
        flags: Union[Unset, List[str]] = UNSET
        if not isinstance(self.flags, Unset):
            flags = []
            for flags_item_data in self.flags:
                flags_item = flags_item_data.value

                flags.append(flags_item)

        overview_settings: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.overview_settings, Unset):
            overview_settings = self.overview_settings.to_dict()

        snowflake_warehouse_name = self.snowflake_warehouse_name
        supported_regions: Union[Unset, List[str]] = UNSET
        if not isinstance(self.supported_regions, Unset):
            supported_regions = []
            for supported_regions_item_data in self.supported_regions:
                supported_regions_item = supported_regions_item_data.value

                supported_regions.append(supported_regions_item)

        enable_sso_oidc = self.enable_sso_oidc
        inactivity_logout_enabled = self.inactivity_logout_enabled
        sms_opt_in_enabled = self.sms_opt_in_enabled
        inactivity_timeout = self.inactivity_timeout
        custom_event_triggers_enabled = self.custom_event_triggers_enabled
        allow_custom_email_configuration = self.allow_custom_email_configuration
        portal_base_url = self.portal_base_url
        ingestion_enabled = self.ingestion_enabled
        timescale_enabled = self.timescale_enabled.isoformat() if self.timescale_enabled else None

        clickhouse_enabled = self.clickhouse_enabled.isoformat() if self.clickhouse_enabled else None

        clickhouse_query_enabled = self.clickhouse_query_enabled
        drop_experimental_query_data = self.drop_experimental_query_data
        event_queries_enabled = self.event_queries_enabled
        snowflake_row_level_security_enabled = self.snowflake_row_level_security_enabled
        clickhouse_analytics_enabled = self.clickhouse_analytics_enabled
        cached_events_enabled = self.cached_events_enabled
        show_get_started_on_overview = self.show_get_started_on_overview
        sidebar_hidden_sections: Union[Unset, List[str]] = UNSET
        if not isinstance(self.sidebar_hidden_sections, Unset):
            sidebar_hidden_sections = self.sidebar_hidden_sections

        design_fleet_manager_white_listed_users = self.design_fleet_manager_white_listed_users
        design_fleet_manager_view_name = self.design_fleet_manager_view_name
        optional_help_link = self.optional_help_link
        show_overview_filters = self.show_overview_filters
        local_mode_enabled = self.local_mode_enabled
        embed_view_enabled = self.embed_view_enabled
        id = self.id
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "plan": plan,
                "name": name,
                "industry": industry,
                "website": website,
                "addressLine1": address_line_1,
                "addressLine2": address_line_2,
                "city": city,
                "state": state,
                "postalCode": postal_code,
                "country": country,
                "addonBillingPeriod": addon_billing_period,
                "invoiceBillingPeriod": invoice_billing_period,
                "trialPeriodEnd": trial_period_end,
                "chargebeeId": chargebee_id,
                "externalTwilioConfig": external_twilio_config,
                "totangoId": totango_id,
                "hubspotId": hubspot_id,
                "snowflakeWarehouseName": snowflake_warehouse_name,
                "timescaleEnabled": timescale_enabled,
                "clickhouseEnabled": clickhouse_enabled,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if pagerduty_info is not UNSET:
            field_dict["pagerdutyInfo"] = pagerduty_info
        if teleop_high_ping_reconnect_behaviors is not UNSET:
            field_dict["teleopHighPingReconnectBehaviors"] = teleop_high_ping_reconnect_behaviors
        if slack_info is not UNSET:
            field_dict["slackInfo"] = slack_info
        if google_info is not UNSET:
            field_dict["googleInfo"] = google_info
        if webhooks_info is not UNSET:
            field_dict["webhooksInfo"] = webhooks_info
        if aws_info is not UNSET:
            field_dict["awsInfo"] = aws_info
        if google_storage_info is not UNSET:
            field_dict["googleStorageInfo"] = google_storage_info
        if looker_info is not UNSET:
            field_dict["lookerInfo"] = looker_info
        if stripe_info is not UNSET:
            field_dict["stripeInfo"] = stripe_info
        if rtc_info is not UNSET:
            field_dict["rtcInfo"] = rtc_info
        if teleop_configuration is not UNSET:
            field_dict["teleopConfiguration"] = teleop_configuration
        if analytics_enabled is not UNSET:
            field_dict["analyticsEnabled"] = analytics_enabled
        if data_export_enabled is not UNSET:
            field_dict["dataExportEnabled"] = data_export_enabled
        if advanced_configuration_enabled is not UNSET:
            field_dict["advancedConfigurationEnabled"] = advanced_configuration_enabled
        if customer_portal_enabled is not UNSET:
            field_dict["customerPortalEnabled"] = customer_portal_enabled
        if stripe_billing_enabled is not UNSET:
            field_dict["stripeBillingEnabled"] = stripe_billing_enabled
        if stripe_subscription_enabled is not UNSET:
            field_dict["stripeSubscriptionEnabled"] = stripe_subscription_enabled
        if mission_planning_enabled is not UNSET:
            field_dict["missionPlanningEnabled"] = mission_planning_enabled
        if billing_info is not UNSET:
            field_dict["billingInfo"] = billing_info
        if s_3_export_enabled is not UNSET:
            field_dict["s3ExportEnabled"] = s_3_export_enabled
        if blob_data_enabled is not UNSET:
            field_dict["blobDataEnabled"] = blob_data_enabled
        if white_label_enabled is not UNSET:
            field_dict["whiteLabelEnabled"] = white_label_enabled
        if viewer_3_d_enabled is not UNSET:
            field_dict["viewer3dEnabled"] = viewer_3_d_enabled
        if adapters_enabled is not UNSET:
            field_dict["adaptersEnabled"] = adapters_enabled
        if white_label_css is not UNSET:
            field_dict["whiteLabelCSS"] = white_label_css
        if demo_mode_enabled is not UNSET:
            field_dict["demoModeEnabled"] = demo_mode_enabled
        if teleop_share_enabled is not UNSET:
            field_dict["teleopShareEnabled"] = teleop_share_enabled
        if bill_estimate_enabled is not UNSET:
            field_dict["billEstimateEnabled"] = bill_estimate_enabled
        if data_retention_enabled is not UNSET:
            field_dict["dataRetentionEnabled"] = data_retention_enabled
        if days_data_retained is not UNSET:
            field_dict["daysDataRetained"] = days_data_retained
        if max_chunk_request_limit is not UNSET:
            field_dict["maxChunkRequestLimit"] = max_chunk_request_limit
        if online_sensitivity is not UNSET:
            field_dict["onlineSensitivity"] = online_sensitivity
        if support_enabled is not UNSET:
            field_dict["supportEnabled"] = support_enabled
        if support_tier is not UNSET:
            field_dict["supportTier"] = support_tier
        if external_id is not UNSET:
            field_dict["externalId"] = external_id
        if custom_tos is not UNSET:
            field_dict["customTos"] = custom_tos
        if teleop_enabled is not UNSET:
            field_dict["teleopEnabled"] = teleop_enabled
        if observability_enabled is not UNSET:
            field_dict["observabilityEnabled"] = observability_enabled
        if share_enabled is not UNSET:
            field_dict["shareEnabled"] = share_enabled
        if annotations_enabled is not UNSET:
            field_dict["annotationsEnabled"] = annotations_enabled
        if diagnostics_enabled is not UNSET:
            field_dict["diagnosticsEnabled"] = diagnostics_enabled
        if ssh_enabled is not UNSET:
            field_dict["sshEnabled"] = ssh_enabled
        if spot_enabled is not UNSET:
            field_dict["spotEnabled"] = spot_enabled
        if file_storage_enabled is not UNSET:
            field_dict["fileStorageEnabled"] = file_storage_enabled
        if role_viewer_enabled is not UNSET:
            field_dict["roleViewerEnabled"] = role_viewer_enabled
        if teams_enabled is not UNSET:
            field_dict["teamsEnabled"] = teams_enabled
        if schedules_enabled is not UNSET:
            field_dict["schedulesEnabled"] = schedules_enabled
        if beta_ui is not UNSET:
            field_dict["betaUI"] = beta_ui
        if paging_enabled is not UNSET:
            field_dict["pagingEnabled"] = paging_enabled
        if stateful_events_enabled is not UNSET:
            field_dict["statefulEventsEnabled"] = stateful_events_enabled
        if billing_enabled is not UNSET:
            field_dict["billingEnabled"] = billing_enabled
        if sms_followers_enabled is not UNSET:
            field_dict["smsFollowersEnabled"] = sms_followers_enabled
        if event_action_interval_enabled is not UNSET:
            field_dict["eventActionIntervalEnabled"] = event_action_interval_enabled
        if task_summaries_enabled is not UNSET:
            field_dict["taskSummariesEnabled"] = task_summaries_enabled
        if default_device_config is not UNSET:
            field_dict["defaultDeviceConfig"] = default_device_config
        if default_stream_throttle_hz is not UNSET:
            field_dict["defaultStreamThrottleHz"] = default_stream_throttle_hz
        if outage_banner_enabled is not UNSET:
            field_dict["outageBannerEnabled"] = outage_banner_enabled
        if new_header_enabled is not UNSET:
            field_dict["newHeaderEnabled"] = new_header_enabled
        if explorations_enabled is not UNSET:
            field_dict["explorationsEnabled"] = explorations_enabled
        if customer_success_ai_enabled is not UNSET:
            field_dict["customerSuccessAIEnabled"] = customer_success_ai_enabled
        if custom_analytics_enabled is not UNSET:
            field_dict["customAnalyticsEnabled"] = custom_analytics_enabled
        if fleet_analytics_enabled is not UNSET:
            field_dict["fleetAnalyticsEnabled"] = fleet_analytics_enabled
        if navigator_enabled is not UNSET:
            field_dict["navigatorEnabled"] = navigator_enabled
        if list_view_enabled is not UNSET:
            field_dict["listViewEnabled"] = list_view_enabled
        if map_view_enabled is not UNSET:
            field_dict["mapViewEnabled"] = map_view_enabled
        if node_graph_page_enabled is not UNSET:
            field_dict["nodeGraphPageEnabled"] = node_graph_page_enabled
        if flags is not UNSET:
            field_dict["flags"] = flags
        if overview_settings is not UNSET:
            field_dict["overviewSettings"] = overview_settings
        if supported_regions is not UNSET:
            field_dict["supportedRegions"] = supported_regions
        if enable_sso_oidc is not UNSET:
            field_dict["enableSsoOidc"] = enable_sso_oidc
        if inactivity_logout_enabled is not UNSET:
            field_dict["inactivityLogoutEnabled"] = inactivity_logout_enabled
        if sms_opt_in_enabled is not UNSET:
            field_dict["smsOptInEnabled"] = sms_opt_in_enabled
        if inactivity_timeout is not UNSET:
            field_dict["inactivityTimeout"] = inactivity_timeout
        if custom_event_triggers_enabled is not UNSET:
            field_dict["customEventTriggersEnabled"] = custom_event_triggers_enabled
        if allow_custom_email_configuration is not UNSET:
            field_dict["allowCustomEmailConfiguration"] = allow_custom_email_configuration
        if portal_base_url is not UNSET:
            field_dict["portalBaseUrl"] = portal_base_url
        if ingestion_enabled is not UNSET:
            field_dict["ingestionEnabled"] = ingestion_enabled
        if clickhouse_query_enabled is not UNSET:
            field_dict["clickhouseQueryEnabled"] = clickhouse_query_enabled
        if drop_experimental_query_data is not UNSET:
            field_dict["dropExperimentalQueryData"] = drop_experimental_query_data
        if event_queries_enabled is not UNSET:
            field_dict["eventQueriesEnabled"] = event_queries_enabled
        if snowflake_row_level_security_enabled is not UNSET:
            field_dict["snowflakeRowLevelSecurityEnabled"] = snowflake_row_level_security_enabled
        if clickhouse_analytics_enabled is not UNSET:
            field_dict["clickhouseAnalyticsEnabled"] = clickhouse_analytics_enabled
        if cached_events_enabled is not UNSET:
            field_dict["cachedEventsEnabled"] = cached_events_enabled
        if show_get_started_on_overview is not UNSET:
            field_dict["showGetStartedOnOverview"] = show_get_started_on_overview
        if sidebar_hidden_sections is not UNSET:
            field_dict["sidebarHiddenSections"] = sidebar_hidden_sections
        if design_fleet_manager_white_listed_users is not UNSET:
            field_dict["designFleetManagerWhiteListedUsers"] = design_fleet_manager_white_listed_users
        if design_fleet_manager_view_name is not UNSET:
            field_dict["designFleetManagerViewName"] = design_fleet_manager_view_name
        if optional_help_link is not UNSET:
            field_dict["optionalHelpLink"] = optional_help_link
        if show_overview_filters is not UNSET:
            field_dict["showOverviewFilters"] = show_overview_filters
        if local_mode_enabled is not UNSET:
            field_dict["localModeEnabled"] = local_mode_enabled
        if embed_view_enabled is not UNSET:
            field_dict["embedViewEnabled"] = embed_view_enabled
        if id is not UNSET:
            field_dict["id"] = id
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if updated_at is not UNSET:
            field_dict["updatedAt"] = updated_at

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.aws_info import AwsInfo
        from ..models.billing_info import BillingInfo
        from ..models.device_configuration_document import DeviceConfigurationDocument
        from ..models.google_info import GoogleInfo
        from ..models.google_storage_info import GoogleStorageInfo
        from ..models.looker_info import LookerInfo
        from ..models.overview_settings import OverviewSettings
        from ..models.pagerduty_info import PagerdutyInfo
        from ..models.rtc_info import RtcInfo
        from ..models.slack_info import SlackInfo
        from ..models.stripe_info import StripeInfo
        from ..models.teleop_high_ping_reconnect_behaviors import TeleopHighPingReconnectBehaviors
        from ..models.user_teleop_configuration import UserTeleopConfiguration
        from ..models.webhooks_info import WebhooksInfo

        d = src_dict.copy()
        plan = OrganizationPlan(d.pop("plan"))

        name = d.pop("name")

        industry = d.pop("industry")

        website = d.pop("website")

        address_line_1 = d.pop("addressLine1")

        address_line_2 = d.pop("addressLine2")

        city = d.pop("city")

        state = d.pop("state")

        postal_code = d.pop("postalCode")

        country = d.pop("country")

        addon_billing_period = OrganizationAddonBillingPeriod(d.pop("addonBillingPeriod"))

        invoice_billing_period = OrganizationInvoiceBillingPeriod(d.pop("invoiceBillingPeriod"))

        description = d.pop("description", UNSET)

        enabled = d.pop("enabled", UNSET)

        _pagerduty_info = d.pop("pagerdutyInfo", UNSET)
        pagerduty_info: Union[Unset, PagerdutyInfo]
        if isinstance(_pagerduty_info, Unset):
            pagerduty_info = UNSET
        else:
            pagerduty_info = PagerdutyInfo.from_dict(_pagerduty_info)

        _teleop_high_ping_reconnect_behaviors = d.pop("teleopHighPingReconnectBehaviors", UNSET)
        teleop_high_ping_reconnect_behaviors: Union[Unset, TeleopHighPingReconnectBehaviors]
        if isinstance(_teleop_high_ping_reconnect_behaviors, Unset):
            teleop_high_ping_reconnect_behaviors = UNSET
        else:
            teleop_high_ping_reconnect_behaviors = TeleopHighPingReconnectBehaviors.from_dict(
                _teleop_high_ping_reconnect_behaviors
            )

        _slack_info = d.pop("slackInfo", UNSET)
        slack_info: Union[Unset, SlackInfo]
        if isinstance(_slack_info, Unset):
            slack_info = UNSET
        else:
            slack_info = SlackInfo.from_dict(_slack_info)

        _google_info = d.pop("googleInfo", UNSET)
        google_info: Union[Unset, GoogleInfo]
        if isinstance(_google_info, Unset):
            google_info = UNSET
        else:
            google_info = GoogleInfo.from_dict(_google_info)

        _webhooks_info = d.pop("webhooksInfo", UNSET)
        webhooks_info: Union[Unset, WebhooksInfo]
        if isinstance(_webhooks_info, Unset):
            webhooks_info = UNSET
        else:
            webhooks_info = WebhooksInfo.from_dict(_webhooks_info)

        _aws_info = d.pop("awsInfo", UNSET)
        aws_info: Union[Unset, AwsInfo]
        if isinstance(_aws_info, Unset):
            aws_info = UNSET
        else:
            aws_info = AwsInfo.from_dict(_aws_info)

        _google_storage_info = d.pop("googleStorageInfo", UNSET)
        google_storage_info: Union[Unset, GoogleStorageInfo]
        if isinstance(_google_storage_info, Unset):
            google_storage_info = UNSET
        else:
            google_storage_info = GoogleStorageInfo.from_dict(_google_storage_info)

        _looker_info = d.pop("lookerInfo", UNSET)
        looker_info: Union[Unset, LookerInfo]
        if isinstance(_looker_info, Unset):
            looker_info = UNSET
        else:
            looker_info = LookerInfo.from_dict(_looker_info)

        _stripe_info = d.pop("stripeInfo", UNSET)
        stripe_info: Union[Unset, StripeInfo]
        if isinstance(_stripe_info, Unset):
            stripe_info = UNSET
        else:
            stripe_info = StripeInfo.from_dict(_stripe_info)

        _rtc_info = d.pop("rtcInfo", UNSET)
        rtc_info: Union[Unset, RtcInfo]
        if isinstance(_rtc_info, Unset):
            rtc_info = UNSET
        else:
            rtc_info = RtcInfo.from_dict(_rtc_info)

        _teleop_configuration = d.pop("teleopConfiguration", UNSET)
        teleop_configuration: Union[Unset, UserTeleopConfiguration]
        if isinstance(_teleop_configuration, Unset):
            teleop_configuration = UNSET
        else:
            teleop_configuration = UserTeleopConfiguration.from_dict(_teleop_configuration)

        _analytics_enabled = d.pop("analyticsEnabled", UNSET)
        analytics_enabled: Union[Unset, None, datetime.datetime]
        if _analytics_enabled is None:
            analytics_enabled = None
        elif isinstance(_analytics_enabled, Unset):
            analytics_enabled = UNSET
        else:
            analytics_enabled = isoparse(_analytics_enabled)

        _data_export_enabled = d.pop("dataExportEnabled", UNSET)
        data_export_enabled: Union[Unset, None, datetime.datetime]
        if _data_export_enabled is None:
            data_export_enabled = None
        elif isinstance(_data_export_enabled, Unset):
            data_export_enabled = UNSET
        else:
            data_export_enabled = isoparse(_data_export_enabled)

        _advanced_configuration_enabled = d.pop("advancedConfigurationEnabled", UNSET)
        advanced_configuration_enabled: Union[Unset, None, datetime.datetime]
        if _advanced_configuration_enabled is None:
            advanced_configuration_enabled = None
        elif isinstance(_advanced_configuration_enabled, Unset):
            advanced_configuration_enabled = UNSET
        else:
            advanced_configuration_enabled = isoparse(_advanced_configuration_enabled)

        _customer_portal_enabled = d.pop("customerPortalEnabled", UNSET)
        customer_portal_enabled: Union[Unset, None, datetime.datetime]
        if _customer_portal_enabled is None:
            customer_portal_enabled = None
        elif isinstance(_customer_portal_enabled, Unset):
            customer_portal_enabled = UNSET
        else:
            customer_portal_enabled = isoparse(_customer_portal_enabled)

        stripe_billing_enabled = d.pop("stripeBillingEnabled", UNSET)

        stripe_subscription_enabled = d.pop("stripeSubscriptionEnabled", UNSET)

        mission_planning_enabled = d.pop("missionPlanningEnabled", UNSET)

        _billing_info = d.pop("billingInfo", UNSET)
        billing_info: Union[Unset, BillingInfo]
        if isinstance(_billing_info, Unset):
            billing_info = UNSET
        else:
            billing_info = BillingInfo.from_dict(_billing_info)

        s_3_export_enabled = d.pop("s3ExportEnabled", UNSET)

        blob_data_enabled = d.pop("blobDataEnabled", UNSET)

        white_label_enabled = d.pop("whiteLabelEnabled", UNSET)

        viewer_3_d_enabled = d.pop("viewer3dEnabled", UNSET)

        adapters_enabled = d.pop("adaptersEnabled", UNSET)

        white_label_css = d.pop("whiteLabelCSS", UNSET)

        demo_mode_enabled = d.pop("demoModeEnabled", UNSET)

        teleop_share_enabled = d.pop("teleopShareEnabled", UNSET)

        bill_estimate_enabled = d.pop("billEstimateEnabled", UNSET)

        _data_retention_enabled = d.pop("dataRetentionEnabled", UNSET)
        data_retention_enabled: Union[Unset, None, datetime.datetime]
        if _data_retention_enabled is None:
            data_retention_enabled = None
        elif isinstance(_data_retention_enabled, Unset):
            data_retention_enabled = UNSET
        else:
            data_retention_enabled = isoparse(_data_retention_enabled)

        days_data_retained = d.pop("daysDataRetained", UNSET)

        max_chunk_request_limit = d.pop("maxChunkRequestLimit", UNSET)

        online_sensitivity = d.pop("onlineSensitivity", UNSET)

        _support_enabled = d.pop("supportEnabled", UNSET)
        support_enabled: Union[Unset, None, datetime.datetime]
        if _support_enabled is None:
            support_enabled = None
        elif isinstance(_support_enabled, Unset):
            support_enabled = UNSET
        else:
            support_enabled = isoparse(_support_enabled)

        _support_tier = d.pop("supportTier", UNSET)
        support_tier: Union[Unset, OrganizationSupportTier]
        if isinstance(_support_tier, Unset):
            support_tier = UNSET
        else:
            support_tier = OrganizationSupportTier(_support_tier)

        _trial_period_end = d.pop("trialPeriodEnd")
        trial_period_end: Optional[datetime.datetime]
        if _trial_period_end is None:
            trial_period_end = None
        else:
            trial_period_end = isoparse(_trial_period_end)

        external_id = d.pop("externalId", UNSET)

        chargebee_id = d.pop("chargebeeId")

        external_twilio_config = d.pop("externalTwilioConfig")

        totango_id = d.pop("totangoId")

        hubspot_id = d.pop("hubspotId")

        custom_tos = d.pop("customTos", UNSET)

        _teleop_enabled = d.pop("teleopEnabled", UNSET)
        teleop_enabled: Union[Unset, None, datetime.datetime]
        if _teleop_enabled is None:
            teleop_enabled = None
        elif isinstance(_teleop_enabled, Unset):
            teleop_enabled = UNSET
        else:
            teleop_enabled = isoparse(_teleop_enabled)

        _observability_enabled = d.pop("observabilityEnabled", UNSET)
        observability_enabled: Union[Unset, None, datetime.datetime]
        if _observability_enabled is None:
            observability_enabled = None
        elif isinstance(_observability_enabled, Unset):
            observability_enabled = UNSET
        else:
            observability_enabled = isoparse(_observability_enabled)

        _share_enabled = d.pop("shareEnabled", UNSET)
        share_enabled: Union[Unset, None, datetime.datetime]
        if _share_enabled is None:
            share_enabled = None
        elif isinstance(_share_enabled, Unset):
            share_enabled = UNSET
        else:
            share_enabled = isoparse(_share_enabled)

        _annotations_enabled = d.pop("annotationsEnabled", UNSET)
        annotations_enabled: Union[Unset, None, datetime.datetime]
        if _annotations_enabled is None:
            annotations_enabled = None
        elif isinstance(_annotations_enabled, Unset):
            annotations_enabled = UNSET
        else:
            annotations_enabled = isoparse(_annotations_enabled)

        _diagnostics_enabled = d.pop("diagnosticsEnabled", UNSET)
        diagnostics_enabled: Union[Unset, None, datetime.datetime]
        if _diagnostics_enabled is None:
            diagnostics_enabled = None
        elif isinstance(_diagnostics_enabled, Unset):
            diagnostics_enabled = UNSET
        else:
            diagnostics_enabled = isoparse(_diagnostics_enabled)

        _ssh_enabled = d.pop("sshEnabled", UNSET)
        ssh_enabled: Union[Unset, None, datetime.datetime]
        if _ssh_enabled is None:
            ssh_enabled = None
        elif isinstance(_ssh_enabled, Unset):
            ssh_enabled = UNSET
        else:
            ssh_enabled = isoparse(_ssh_enabled)

        _spot_enabled = d.pop("spotEnabled", UNSET)
        spot_enabled: Union[Unset, None, datetime.datetime]
        if _spot_enabled is None:
            spot_enabled = None
        elif isinstance(_spot_enabled, Unset):
            spot_enabled = UNSET
        else:
            spot_enabled = isoparse(_spot_enabled)

        file_storage_enabled = d.pop("fileStorageEnabled", UNSET)

        role_viewer_enabled = d.pop("roleViewerEnabled", UNSET)

        teams_enabled = d.pop("teamsEnabled", UNSET)

        schedules_enabled = d.pop("schedulesEnabled", UNSET)

        beta_ui = d.pop("betaUI", UNSET)

        paging_enabled = d.pop("pagingEnabled", UNSET)

        stateful_events_enabled = d.pop("statefulEventsEnabled", UNSET)

        billing_enabled = d.pop("billingEnabled", UNSET)

        sms_followers_enabled = d.pop("smsFollowersEnabled", UNSET)

        event_action_interval_enabled = d.pop("eventActionIntervalEnabled", UNSET)

        task_summaries_enabled = d.pop("taskSummariesEnabled", UNSET)

        _default_device_config = d.pop("defaultDeviceConfig", UNSET)
        default_device_config: Union[Unset, DeviceConfigurationDocument]
        if isinstance(_default_device_config, Unset):
            default_device_config = UNSET
        else:
            default_device_config = DeviceConfigurationDocument.from_dict(_default_device_config)

        default_stream_throttle_hz = d.pop("defaultStreamThrottleHz", UNSET)

        outage_banner_enabled = d.pop("outageBannerEnabled", UNSET)

        new_header_enabled = d.pop("newHeaderEnabled", UNSET)

        explorations_enabled = d.pop("explorationsEnabled", UNSET)

        customer_success_ai_enabled = d.pop("customerSuccessAIEnabled", UNSET)

        custom_analytics_enabled = d.pop("customAnalyticsEnabled", UNSET)

        fleet_analytics_enabled = d.pop("fleetAnalyticsEnabled", UNSET)

        navigator_enabled = d.pop("navigatorEnabled", UNSET)

        list_view_enabled = d.pop("listViewEnabled", UNSET)

        map_view_enabled = d.pop("mapViewEnabled", UNSET)

        node_graph_page_enabled = d.pop("nodeGraphPageEnabled", UNSET)

        flags = []
        _flags = d.pop("flags", UNSET)
        for flags_item_data in _flags or []:
            flags_item = OrganizationFlagsItem(flags_item_data)

            flags.append(flags_item)

        _overview_settings = d.pop("overviewSettings", UNSET)
        overview_settings: Union[Unset, OverviewSettings]
        if isinstance(_overview_settings, Unset):
            overview_settings = UNSET
        else:
            overview_settings = OverviewSettings.from_dict(_overview_settings)

        snowflake_warehouse_name = d.pop("snowflakeWarehouseName")

        supported_regions = []
        _supported_regions = d.pop("supportedRegions", UNSET)
        for supported_regions_item_data in _supported_regions or []:
            supported_regions_item = OrganizationSupportedRegionsItem(supported_regions_item_data)

            supported_regions.append(supported_regions_item)

        enable_sso_oidc = d.pop("enableSsoOidc", UNSET)

        inactivity_logout_enabled = d.pop("inactivityLogoutEnabled", UNSET)

        sms_opt_in_enabled = d.pop("smsOptInEnabled", UNSET)

        inactivity_timeout = d.pop("inactivityTimeout", UNSET)

        custom_event_triggers_enabled = d.pop("customEventTriggersEnabled", UNSET)

        allow_custom_email_configuration = d.pop("allowCustomEmailConfiguration", UNSET)

        portal_base_url = d.pop("portalBaseUrl", UNSET)

        ingestion_enabled = d.pop("ingestionEnabled", UNSET)

        _timescale_enabled = d.pop("timescaleEnabled")
        timescale_enabled: Optional[datetime.datetime]
        if _timescale_enabled is None:
            timescale_enabled = None
        else:
            timescale_enabled = isoparse(_timescale_enabled)

        _clickhouse_enabled = d.pop("clickhouseEnabled")
        clickhouse_enabled: Optional[datetime.datetime]
        if _clickhouse_enabled is None:
            clickhouse_enabled = None
        else:
            clickhouse_enabled = isoparse(_clickhouse_enabled)

        clickhouse_query_enabled = d.pop("clickhouseQueryEnabled", UNSET)

        drop_experimental_query_data = d.pop("dropExperimentalQueryData", UNSET)

        event_queries_enabled = d.pop("eventQueriesEnabled", UNSET)

        snowflake_row_level_security_enabled = d.pop("snowflakeRowLevelSecurityEnabled", UNSET)

        clickhouse_analytics_enabled = d.pop("clickhouseAnalyticsEnabled", UNSET)

        cached_events_enabled = d.pop("cachedEventsEnabled", UNSET)

        show_get_started_on_overview = d.pop("showGetStartedOnOverview", UNSET)

        sidebar_hidden_sections = cast(List[str], d.pop("sidebarHiddenSections", UNSET))

        design_fleet_manager_white_listed_users = d.pop("designFleetManagerWhiteListedUsers", UNSET)

        design_fleet_manager_view_name = d.pop("designFleetManagerViewName", UNSET)

        optional_help_link = d.pop("optionalHelpLink", UNSET)

        show_overview_filters = d.pop("showOverviewFilters", UNSET)

        local_mode_enabled = d.pop("localModeEnabled", UNSET)

        embed_view_enabled = d.pop("embedViewEnabled", UNSET)

        id = d.pop("id", UNSET)

        _created_at = d.pop("createdAt", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _updated_at = d.pop("updatedAt", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        organization = cls(
            plan=plan,
            name=name,
            industry=industry,
            website=website,
            address_line_1=address_line_1,
            address_line_2=address_line_2,
            city=city,
            state=state,
            postal_code=postal_code,
            country=country,
            addon_billing_period=addon_billing_period,
            invoice_billing_period=invoice_billing_period,
            description=description,
            enabled=enabled,
            pagerduty_info=pagerduty_info,
            teleop_high_ping_reconnect_behaviors=teleop_high_ping_reconnect_behaviors,
            slack_info=slack_info,
            google_info=google_info,
            webhooks_info=webhooks_info,
            aws_info=aws_info,
            google_storage_info=google_storage_info,
            looker_info=looker_info,
            stripe_info=stripe_info,
            rtc_info=rtc_info,
            teleop_configuration=teleop_configuration,
            analytics_enabled=analytics_enabled,
            data_export_enabled=data_export_enabled,
            advanced_configuration_enabled=advanced_configuration_enabled,
            customer_portal_enabled=customer_portal_enabled,
            stripe_billing_enabled=stripe_billing_enabled,
            stripe_subscription_enabled=stripe_subscription_enabled,
            mission_planning_enabled=mission_planning_enabled,
            billing_info=billing_info,
            s_3_export_enabled=s_3_export_enabled,
            blob_data_enabled=blob_data_enabled,
            white_label_enabled=white_label_enabled,
            viewer_3_d_enabled=viewer_3_d_enabled,
            adapters_enabled=adapters_enabled,
            white_label_css=white_label_css,
            demo_mode_enabled=demo_mode_enabled,
            teleop_share_enabled=teleop_share_enabled,
            bill_estimate_enabled=bill_estimate_enabled,
            data_retention_enabled=data_retention_enabled,
            days_data_retained=days_data_retained,
            max_chunk_request_limit=max_chunk_request_limit,
            online_sensitivity=online_sensitivity,
            support_enabled=support_enabled,
            support_tier=support_tier,
            trial_period_end=trial_period_end,
            external_id=external_id,
            chargebee_id=chargebee_id,
            external_twilio_config=external_twilio_config,
            totango_id=totango_id,
            hubspot_id=hubspot_id,
            custom_tos=custom_tos,
            teleop_enabled=teleop_enabled,
            observability_enabled=observability_enabled,
            share_enabled=share_enabled,
            annotations_enabled=annotations_enabled,
            diagnostics_enabled=diagnostics_enabled,
            ssh_enabled=ssh_enabled,
            spot_enabled=spot_enabled,
            file_storage_enabled=file_storage_enabled,
            role_viewer_enabled=role_viewer_enabled,
            teams_enabled=teams_enabled,
            schedules_enabled=schedules_enabled,
            beta_ui=beta_ui,
            paging_enabled=paging_enabled,
            stateful_events_enabled=stateful_events_enabled,
            billing_enabled=billing_enabled,
            sms_followers_enabled=sms_followers_enabled,
            event_action_interval_enabled=event_action_interval_enabled,
            task_summaries_enabled=task_summaries_enabled,
            default_device_config=default_device_config,
            default_stream_throttle_hz=default_stream_throttle_hz,
            outage_banner_enabled=outage_banner_enabled,
            new_header_enabled=new_header_enabled,
            explorations_enabled=explorations_enabled,
            customer_success_ai_enabled=customer_success_ai_enabled,
            custom_analytics_enabled=custom_analytics_enabled,
            fleet_analytics_enabled=fleet_analytics_enabled,
            navigator_enabled=navigator_enabled,
            list_view_enabled=list_view_enabled,
            map_view_enabled=map_view_enabled,
            node_graph_page_enabled=node_graph_page_enabled,
            flags=flags,
            overview_settings=overview_settings,
            snowflake_warehouse_name=snowflake_warehouse_name,
            supported_regions=supported_regions,
            enable_sso_oidc=enable_sso_oidc,
            inactivity_logout_enabled=inactivity_logout_enabled,
            sms_opt_in_enabled=sms_opt_in_enabled,
            inactivity_timeout=inactivity_timeout,
            custom_event_triggers_enabled=custom_event_triggers_enabled,
            allow_custom_email_configuration=allow_custom_email_configuration,
            portal_base_url=portal_base_url,
            ingestion_enabled=ingestion_enabled,
            timescale_enabled=timescale_enabled,
            clickhouse_enabled=clickhouse_enabled,
            clickhouse_query_enabled=clickhouse_query_enabled,
            drop_experimental_query_data=drop_experimental_query_data,
            event_queries_enabled=event_queries_enabled,
            snowflake_row_level_security_enabled=snowflake_row_level_security_enabled,
            clickhouse_analytics_enabled=clickhouse_analytics_enabled,
            cached_events_enabled=cached_events_enabled,
            show_get_started_on_overview=show_get_started_on_overview,
            sidebar_hidden_sections=sidebar_hidden_sections,
            design_fleet_manager_white_listed_users=design_fleet_manager_white_listed_users,
            design_fleet_manager_view_name=design_fleet_manager_view_name,
            optional_help_link=optional_help_link,
            show_overview_filters=show_overview_filters,
            local_mode_enabled=local_mode_enabled,
            embed_view_enabled=embed_view_enabled,
            id=id,
            created_at=created_at,
            updated_at=updated_at,
        )

        organization.additional_properties = d
        return organization

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# flake8: noqa

from .annotation_template import AnnotationTemplate
from .annotation import Annotation
from .annotation_template_list_result import AnnotationTemplateListResult
from .annotation_field import AnnotationField



from formant.protos.model.v1 import math_pb2
from formant.sdk.agent.v1.localization.utils import validate_type, get_ros_module


class Vector3:
    @classmethod
    def from_ros(cls, vector3):
        msgs = get_ros_module("geometry_msgs.msg")
        validate_type(vector3, msgs.Vector3)
        return cls(vector3.x, vector3.y, vector3.z)

    @classmethod
    def from_ros_point(cls, point):
        msgs = get_ros_module("geometry_msgs.msg")
        validate_type(point, msgs.Point)
        return cls(point.x, point.y, point.z)

    @classmethod
    def from_xy(
        cls,
        x,  # type: float
        y,  # type: float
    ):
        return cls(x, y, 0)

    @classmethod
    def identity(cls):
        return cls(0, 0, 0)

    def __init__(
        self,
        x,  # type: float
        y,  # type: float
        z,  # type: float
    ):
        self.x = x
        self.y = y
        self.z = z

    def to_proto(self):
        return math_pb2.Vector3(x=self.x, y=self.y, z=self.z)




from formant.sdk.cloud.v2.formant_admin_api_client.models import TransformTreeViewConfiguration, ViewLayoutType, ViewListResponse, LocalizationViewConfiguration, LocationViewport, PartialViewTags, Location, ViewConfiguration, ViewConfigurationType, Filter, Camera, PartialViewLayoutType, ViewTags, ImageViewConfiguration, ImageViewConfigurationMode, View, NumericViewConfiguration, PointCloudViewConfiguration, PartialView, LocationViewConfiguration, Vector3, LocationViewConfigurationBasemap, BitsetViewConfiguration
from formant.sdk.cloud.v2.formant_admin_api_client.api.view import view_controller_get_one, view_controller_get_all, view_controller_patch
from formant.sdk.cloud.v2.formant_admin_api_client.types import *



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

import attr

from ..models.device_details_type import DeviceDetailsType

if TYPE_CHECKING:
    from ..models.device_details_tags import DeviceDetailsTags


T = TypeVar("T", bound="DeviceDetails")


@attr.s(auto_attribs=True)
class DeviceDetails:
    """
    Attributes:
        id (str):
        name (str):
        type (DeviceDetailsType):
        tags (DeviceDetailsTags):
        enabled (bool):
        public_key (str):
    """

    id: str
    name: str
    type: DeviceDetailsType
    tags: "DeviceDetailsTags"
    enabled: bool
    public_key: str
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        name = self.name
        type = self.type.value

        tags = self.tags.to_dict()

        enabled = self.enabled
        public_key = self.public_key

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "name": name,
                "type": type,
                "tags": tags,
                "enabled": enabled,
                "publicKey": public_key,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.device_details_tags import DeviceDetailsTags

        d = src_dict.copy()
        id = d.pop("id")

        name = d.pop("name")

        type = DeviceDetailsType(d.pop("type"))

        tags = DeviceDetailsTags.from_dict(d.pop("tags"))

        enabled = d.pop("enabled")

        public_key = d.pop("publicKey")

        device_details = cls(
            id=id,
            name=name,
            type=type,
            tags=tags,
            enabled=enabled,
            public_key=public_key,
        )

        device_details.additional_properties = d
        return device_details

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar

import attr
from dateutil.parser import isoparse

T = TypeVar("T", bound="GoogleInfo")


@attr.s(auto_attribs=True)
class GoogleInfo:
    """
    Attributes:
        refresh_token (str):
        access_token (str):
        expiration_date (datetime.datetime):
        scope (str):
    """

    refresh_token: str
    access_token: str
    expiration_date: datetime.datetime
    scope: str
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        refresh_token = self.refresh_token
        access_token = self.access_token
        expiration_date = self.expiration_date.isoformat()

        scope = self.scope

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "refreshToken": refresh_token,
                "accessToken": access_token,
                "expirationDate": expiration_date,
                "scope": scope,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        refresh_token = d.pop("refreshToken")

        access_token = d.pop("accessToken")

        expiration_date = isoparse(d.pop("expirationDate"))

        scope = d.pop("scope")

        google_info = cls(
            refresh_token=refresh_token,
            access_token=access_token,
            expiration_date=expiration_date,
            scope=scope,
        )

        google_info.additional_properties = d
        return google_info

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.layout_module_configuration_module_type import LayoutModuleConfigurationModuleType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.location_module_parameters import LocationModuleParameters


T = TypeVar("T", bound="LayoutModuleConfiguration")


@attr.s(auto_attribs=True)
class LayoutModuleConfiguration:
    """
    Attributes:
        module_type (Union[Unset, LayoutModuleConfigurationModuleType]):
        parameters (Union[Unset, LocationModuleParameters]):
        custom_name (Union[Unset, str]):
        custom_visualization_url (Union[Unset, str]):
        debug (Union[Unset, bool]):
        show_only_current_values (Union[Unset, bool]):
        streams (Union[Unset, List[str]]):
        visible (Union[Unset, bool]):
    """

    module_type: Union[Unset, LayoutModuleConfigurationModuleType] = UNSET
    parameters: Union[Unset, "LocationModuleParameters"] = UNSET
    custom_name: Union[Unset, str] = UNSET
    custom_visualization_url: Union[Unset, str] = UNSET
    debug: Union[Unset, bool] = UNSET
    show_only_current_values: Union[Unset, bool] = UNSET
    streams: Union[Unset, List[str]] = UNSET
    visible: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        module_type: Union[Unset, str] = UNSET
        if not isinstance(self.module_type, Unset):
            module_type = self.module_type.value

        parameters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.parameters, Unset):
            parameters = self.parameters.to_dict()

        custom_name = self.custom_name
        custom_visualization_url = self.custom_visualization_url
        debug = self.debug
        show_only_current_values = self.show_only_current_values
        streams: Union[Unset, List[str]] = UNSET
        if not isinstance(self.streams, Unset):
            streams = self.streams

        visible = self.visible

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if module_type is not UNSET:
            field_dict["moduleType"] = module_type
        if parameters is not UNSET:
            field_dict["parameters"] = parameters
        if custom_name is not UNSET:
            field_dict["customName"] = custom_name
        if custom_visualization_url is not UNSET:
            field_dict["customVisualizationUrl"] = custom_visualization_url
        if debug is not UNSET:
            field_dict["debug"] = debug
        if show_only_current_values is not UNSET:
            field_dict["showOnlyCurrentValues"] = show_only_current_values
        if streams is not UNSET:
            field_dict["streams"] = streams
        if visible is not UNSET:
            field_dict["visible"] = visible

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.location_module_parameters import LocationModuleParameters

        d = src_dict.copy()
        _module_type = d.pop("moduleType", UNSET)
        module_type: Union[Unset, LayoutModuleConfigurationModuleType]
        if isinstance(_module_type, Unset):
            module_type = UNSET
        else:
            module_type = LayoutModuleConfigurationModuleType(_module_type)

        _parameters = d.pop("parameters", UNSET)
        parameters: Union[Unset, LocationModuleParameters]
        if isinstance(_parameters, Unset):
            parameters = UNSET
        else:
            parameters = LocationModuleParameters.from_dict(_parameters)

        custom_name = d.pop("customName", UNSET)

        custom_visualization_url = d.pop("customVisualizationUrl", UNSET)

        debug = d.pop("debug", UNSET)

        show_only_current_values = d.pop("showOnlyCurrentValues", UNSET)

        streams = cast(List[str], d.pop("streams", UNSET))

        visible = d.pop("visible", UNSET)

        layout_module_configuration = cls(
            module_type=module_type,
            parameters=parameters,
            custom_name=custom_name,
            custom_visualization_url=custom_visualization_url,
            debug=debug,
            show_only_current_values=show_only_current_values,
            streams=streams,
            visible=visible,
        )

        layout_module_configuration.additional_properties = d
        return layout_module_configuration

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()


setuptools.setup(
    name="formant",
    version="1.184.2",
    author="Formant",
    author_email="eng@formant.io",
    description="Formant python package",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://formant.io",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        ## IMPORTANT!! If you need to change any of this
        ## you will need to test the python SDK for every targets we support
        ## which means Ubuntu 18, 20, 22 and for both python 2 and python 3
        ## Specifically the protobuf versions need to account for
        ## Formant grpc protos generationa as well as external library dependencies
        ## sych the googleapis or grpcio own requirements.
        "grpcio>=1.37.0",
        "grpcio-status>=1.37.0",
        'protobuf==3.17.3; python_version<"3.0.0"',
        'protobuf>=3.15.8, <=3.19.5; python_version>="3.0.0"',
        "typing-extensions>=3.7.4.2",
        "requests>=2.25.1",
        "python-lzf>=0.2.4",
        "jsonschema>=1.0.0",
        "python-dateutil",
        "httpx; python_version>='3.0.0'",
        "numpy>=1.16.6",
        "wheel>=0.37.1",
    ],
    python_requires=">=2.7",
)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.event_trigger import EventTrigger


T = TypeVar("T", bound="UpdatedEventTriggerResponse")


@attr.s(auto_attribs=True)
class UpdatedEventTriggerResponse:
    """
    Attributes:
        event_triggers (Union[Unset, None, List['EventTrigger']]):
        last_updated (Union[Unset, int]):
    """

    event_triggers: Union[Unset, None, List["EventTrigger"]] = UNSET
    last_updated: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        event_triggers: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.event_triggers, Unset):
            if self.event_triggers is None:
                event_triggers = None
            else:
                event_triggers = []
                for event_triggers_item_data in self.event_triggers:
                    event_triggers_item = event_triggers_item_data.to_dict()

                    event_triggers.append(event_triggers_item)

        last_updated = self.last_updated

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if event_triggers is not UNSET:
            field_dict["eventTriggers"] = event_triggers
        if last_updated is not UNSET:
            field_dict["lastUpdated"] = last_updated

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.event_trigger import EventTrigger

        d = src_dict.copy()
        event_triggers = []
        _event_triggers = d.pop("eventTriggers", UNSET)
        for event_triggers_item_data in _event_triggers or []:
            event_triggers_item = EventTrigger.from_dict(event_triggers_item_data)

            event_triggers.append(event_triggers_item)

        last_updated = d.pop("lastUpdated", UNSET)

        updated_event_trigger_response = cls(
            event_triggers=event_triggers,
            last_updated=last_updated,
        )

        updated_event_trigger_response.additional_properties = d
        return updated_event_trigger_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.device_configuration_document import DeviceConfigurationDocument
    from ..models.device_configuration_template_tags import DeviceConfigurationTemplateTags


T = TypeVar("T", bound="DeviceConfigurationTemplate")


@attr.s(auto_attribs=True)
class DeviceConfigurationTemplate:
    """
    Attributes:
        name (str):
        document (DeviceConfigurationDocument):
        organization_id (Union[Unset, str]):
        enabled (Union[Unset, bool]):
        id (Union[Unset, str]):
        created_at (Union[Unset, datetime.datetime]):
        updated_at (Union[Unset, datetime.datetime]):
        tags (Union[Unset, DeviceConfigurationTemplateTags]):
    """

    name: str
    document: "DeviceConfigurationDocument"
    organization_id: Union[Unset, str] = UNSET
    enabled: Union[Unset, bool] = UNSET
    id: Union[Unset, str] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    tags: Union[Unset, "DeviceConfigurationTemplateTags"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        document = self.document.to_dict()

        organization_id = self.organization_id
        enabled = self.enabled
        id = self.id
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        tags: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "document": document,
            }
        )
        if organization_id is not UNSET:
            field_dict["organizationId"] = organization_id
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if id is not UNSET:
            field_dict["id"] = id
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if updated_at is not UNSET:
            field_dict["updatedAt"] = updated_at
        if tags is not UNSET:
            field_dict["tags"] = tags

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.device_configuration_document import DeviceConfigurationDocument
        from ..models.device_configuration_template_tags import DeviceConfigurationTemplateTags

        d = src_dict.copy()
        name = d.pop("name")

        document = DeviceConfigurationDocument.from_dict(d.pop("document"))

        organization_id = d.pop("organizationId", UNSET)

        enabled = d.pop("enabled", UNSET)

        id = d.pop("id", UNSET)

        _created_at = d.pop("createdAt", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _updated_at = d.pop("updatedAt", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        _tags = d.pop("tags", UNSET)
        tags: Union[Unset, DeviceConfigurationTemplateTags]
        if isinstance(_tags, Unset):
            tags = UNSET
        else:
            tags = DeviceConfigurationTemplateTags.from_dict(_tags)

        device_configuration_template = cls(
            name=name,
            document=document,
            organization_id=organization_id,
            enabled=enabled,
            id=id,
            created_at=created_at,
            updated_at=updated_at,
            tags=tags,
        )

        device_configuration_template.additional_properties = d
        return device_configuration_template

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties




from formant.sdk.cloud.v2.formant_query_api_client.models import IntervalQuery, IntervalQueryInterval, IsoDateListResponse, IntervalQueryTypesItem, IntervalQueryAggregate
from formant.sdk.cloud.v2.formant_query_api_client.api.presence import presence_controller_count
from formant.sdk.cloud.v2.formant_query_api_client.types import *



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="KernelInfo")


@attr.s(auto_attribs=True)
class KernelInfo:
    """
    Attributes:
        version (Union[Unset, str]):
        release (Union[Unset, str]):
        architecture (Union[Unset, str]):
    """

    version: Union[Unset, str] = UNSET
    release: Union[Unset, str] = UNSET
    architecture: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        version = self.version
        release = self.release
        architecture = self.architecture

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if version is not UNSET:
            field_dict["version"] = version
        if release is not UNSET:
            field_dict["release"] = release
        if architecture is not UNSET:
            field_dict["architecture"] = architecture

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        version = d.pop("version", UNSET)

        release = d.pop("release", UNSET)

        architecture = d.pop("architecture", UNSET)

        kernel_info = cls(
            version=version,
            release=release,
            architecture=architecture,
        )

        kernel_info.additional_properties = d
        return kernel_info

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar

import attr
from dateutil.parser import isoparse

T = TypeVar("T", bound="DeviceCredentials")


@attr.s(auto_attribs=True)
class DeviceCredentials:
    """
    Attributes:
        access_key_id (str):
        secret_access_key (str):
        session_token (str):
        expiration (datetime.datetime):
        s_3_region (str):
        s_3_upload_bucket (str):
    """

    access_key_id: str
    secret_access_key: str
    session_token: str
    expiration: datetime.datetime
    s_3_region: str
    s_3_upload_bucket: str
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        access_key_id = self.access_key_id
        secret_access_key = self.secret_access_key
        session_token = self.session_token
        expiration = self.expiration.isoformat()

        s_3_region = self.s_3_region
        s_3_upload_bucket = self.s_3_upload_bucket

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "accessKeyId": access_key_id,
                "secretAccessKey": secret_access_key,
                "sessionToken": session_token,
                "expiration": expiration,
                "s3Region": s_3_region,
                "s3UploadBucket": s_3_upload_bucket,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        access_key_id = d.pop("accessKeyId")

        secret_access_key = d.pop("secretAccessKey")

        session_token = d.pop("sessionToken")

        expiration = isoparse(d.pop("expiration"))

        s_3_region = d.pop("s3Region")

        s_3_upload_bucket = d.pop("s3UploadBucket")

        device_credentials = cls(
            access_key_id=access_key_id,
            secret_access_key=secret_access_key,
            session_token=session_token,
            expiration=expiration,
            s_3_region=s_3_region,
            s_3_upload_bucket=s_3_upload_bucket,
        )

        device_credentials.additional_properties = d
        return device_credentials

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties




from formant.sdk.cloud.v2.formant_admin_api_client.models import DeviceTeleopRosStreamConfiguration, DeviceConfigurationTemplateListResponse, DeviceStreamConfiguration, DeviceTags, OnDemandBuffer, OnDemandStreamPresence, DeviceOfflineEventType, OnDemandState, DeviceTelemetryConfiguration, DeviceTeleopConfiguration, DeviceStreamRosLocalizationConfigurationType, DeviceTeleopHardwareStreamConfigurationRtcStreamType, DeviceOfflineEvent, Network, DeviceType, DeviceConfiguration, DeviceStreamHardwareConfigurationType, DeviceStreamConfigurationTags, TeleopJoystickAxisConfiguration, HwInfo, DeviceBlobData, DeviceStreamRosLocalizationConfiguration, DeviceOfflineEventSeverity, OnDemandBufferBufferType, DeviceStreamCustomConfiguration, DeviceStreamRosTopicConfigurationType, DeviceApplicationConfigurationConfigurationMap, RtcInfoRtcIceTransportPoliciesItem, DeviceStateEnv, AudioDevice, VideoDevice, DeviceFollower, CommandProgress, DeviceQuery, DeviceStreamDirectoryWatchConfiguration, DeviceTeleopHardwareStreamConfigurationMode, DeviceOnlineEventMetadata, DeviceResourcesConfiguration, DeviceTeleopRosStreamConfigurationAudioCodec, DeviceTeleopCustomStreamConfigurationRtcStreamType, DeviceOnlineEvent, NodeInfo, DeviceState, DeviceDetails, DeviceStreamHardwareConfigurationQuality, DeviceTeleopCustomStreamConfigurationNumericControlVisualization, TeleopJoystickAxisConfigurationDimension, DeviceTeleopRosStreamConfigurationTopicType, DeviceScopeTypesItem, DeviceStreamFileTailConfigurationFileFormat, DeviceDetailsTags, DeviceConfigurationTemplateTags, DeviceStreamDirectoryWatchConfigurationFileType, DeviceStreamHardwareConfiguration, PartialDevice, DeviceStreamRosTransformTreeConfiguration, PartialDeviceType, ScopeFilterTypesItem, DeviceRosState, DeviceDetailsType, PartialDeviceTags, DeviceProvisioningRequest, DeviceProvisioning, OsInfo, DeviceStreamHardwareConfigurationHardwareType, DevicePortForwardingConfiguration, DeviceTeleopRosStreamConfigurationQuality, OnDemandPresenceTimeRange, DeviceConfigurationDocument, OnDemandPresenceStreamItemGroup, OnvifDevice, DeviceListResponse, ScopeFilter, DeviceDetailsListResponse, RtcInfo, DeviceStreamTransformConfiguration, DeviceTeleopRosStreamConfigurationNumericControlVisualization, RosTopic, NetworkInfo, DeviceStreamFileTailConfiguration, DeviceStreamRosTopicConfiguration, ValidationConfiguration, KernelInfo, DeviceStreamFileTailConfigurationType, DeviceRosConfiguration, DeviceTeleopRosStreamConfigurationMode, DeviceTeleopHardwareStreamConfigurationQuality, RtcInfoRtcIceServerProtocol, DeviceOnlineEventType, DeviceStreamRosTransformTreeConfigurationType, DeviceStreamDirectoryWatchConfigurationType, DeviceStreamConfigurationQuality, DeviceDiskConfiguration, DeviceQueryType, DeviceReportedConfigurationState, DeviceTeleopHardwareStreamConfigurationHardwareType, DeviceTeleopCustomStreamConfigurationMode, DeviceOnlineEventStreamType, DeviceOnlineEventSeverity, DeviceApplicationConfiguration, DeviceOfflineEventMetadata, Device, DeviceScope, TeleopJoystickConfigurationPosition, TeleopJoystickConfiguration, DeviceOfflineEventTags, DeviceConfigurationTemplate, DeviceTeleopCustomStreamConfigurationQuality, OnDemandPresenceStreamItemGroupDatapointType, DeviceOfflineEventStreamType, TeleopViewConfiguration, AdapterConfiguration, DeviceCredentials, DeviceOnlineEventTags, DeviceTeleopHardwareStreamConfiguration, DeviceStreamCustomConfigurationType, DeviceDiagnosticsConfiguration, DeviceTeleopCustomStreamConfiguration, DeviceConfigurationDocumentTags
from formant.sdk.cloud.v2.formant_admin_api_client.api.device import device_controller_query, device_controller_patch, device_controller_get_one, device_controller_get_configuration, device_controller_post_configuration
from formant.sdk.cloud.v2.formant_admin_api_client.types import *
from formant.sdk.cloud.v2.formant_admin_api_client.types import Optional



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="StatefulTriggerConfiguration")


@attr.s(auto_attribs=True)
class StatefulTriggerConfiguration:
    """
    Attributes:
        emit_on_entering_state (Union[Unset, bool]):
        emit_on_leaving_state (Union[Unset, bool]):
    """

    emit_on_entering_state: Union[Unset, bool] = UNSET
    emit_on_leaving_state: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        emit_on_entering_state = self.emit_on_entering_state
        emit_on_leaving_state = self.emit_on_leaving_state

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if emit_on_entering_state is not UNSET:
            field_dict["emitOnEnteringState"] = emit_on_entering_state
        if emit_on_leaving_state is not UNSET:
            field_dict["emitOnLeavingState"] = emit_on_leaving_state

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        emit_on_entering_state = d.pop("emitOnEnteringState", UNSET)

        emit_on_leaving_state = d.pop("emitOnLeavingState", UNSET)

        stateful_trigger_configuration = cls(
            emit_on_entering_state=emit_on_entering_state,
            emit_on_leaving_state=emit_on_leaving_state,
        )

        stateful_trigger_configuration.additional_properties = d
        return stateful_trigger_configuration

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Optional, List, Dict  # noqa: F401
import ntpath
import os
import requests
import sys
import time
import datetime
from dateutil.tz import tzutc
import copy

FORMANT_REQUEST_ON_DEMAND_DATA_COMMAND_NAME = "formant.demand_data"


def get_current_isodate():
    return datetime.datetime.now(tz=tzutc()).isoformat()


def get_timestamp_str(
    dt,  # type: datetime.datetime
):
    return str(int(dt.timestamp()))


def timestamp_to_datetime_utc(timestamp):
    try:
        my_time = datetime.datetime.fromtimestamp(
            float(timestamp),
            datetime.timezone.utc
        ).isoformat()
    except (ValueError, TypeError) as e:
        my_time = timestamp

    return my_time


class Client:
    """
    A client for interacting with the Formant Cloud. There are methods for:
    - Ingesting telemetry datapoints for device(s)
    - Query telemetry datapoints
    - Query stream(s) last known value
    - Create intervention requests
    - Create intervention responses
    Requires service account credentials (environment variables):
    - FORMANT_EMAIL
    - FORMANT_PASSWORD
    """

    def __init__(
        self,
        admin_api="https://api.formant.io/v1/admin",
        ingest_api="https://api.formant.io/v1/ingest",
        query_api="https://api.formant.io/v1/queries",
    ):
        self._admin_api = admin_api
        self._ingest_api = ingest_api
        self._query_api = query_api

        self._email = os.getenv("FORMANT_EMAIL")
        self._password = os.getenv("FORMANT_PASSWORD")
        if self._email is None:
            raise ValueError("Missing FORMANT_EMAIL environment variable")
        if self._password is None:
            raise ValueError("Missing FORMANT_PASSWORD environment variable")

        self._headers = {
            "Content-Type": "application/json",
            "App-ID": "formant/python-cloud-sdk",
        }

        self._token = None
        self._token_expiry = 0
        self._organization_id = None
        self._user_id = None

    def get_user_id(self):
        if self._user_id is None:
            self._authenticate()
        return self._user_id

    def get_organization_id(self):
        if self._organization_id is None:
            self._authenticate()
        return self._organization_id

    def ingest(self, params):
        """Administrator credentials required.
        Example ingestion params:
        {
            deviceId: "ced176ab-f223-4466-b958-ff8d35261529",
            name: "engine_temp",
            type: "numeric",
            tags: {"location":"sf"},
            points: [...],
        }
        """

        def call(token):
            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            response = requests.post(
                "%s/batch" % self._ingest_api, headers=headers, json=params
            )
            response.raise_for_status()

        self._authenticate_request(call)

    def get_organization(
        self,
    ):
        def call(token):
            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            response = requests.get(
                "%s/organizations/%s" % (self._admin_api, self.get_organization_id()),
                headers=headers,
            )
            response.raise_for_status()
            return response.json()

        return self._authenticate_request(call)

    def query(self, params):
        """Example query params (only start and end time are required):
        {
            start: "2021-01-01T01:00:00.000Z",
            end: "2021-01-01T02:00:00.000Z",
            deviceIds: ["99e8ee37-0a27-4a11-bba2-521facabefa3"],
            names: ["engine_temp"],
            types: ["numeric"],
            tags: {"location":["sf","la"]},
            notNames: ["speed"],
        }
        """

        def call(token):
            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            # enable pagination by default
            # TODO: support for aggregate queries
            query = copy.deepcopy(params)
            nextToken = None if len(query.get("aggregate", "")) > 0 else "true"
            result = {"items": []}
            while True:
                if nextToken is not None:
                    query["next"] = nextToken
                response = requests.post(
                    "%s/queries" % self._query_api, headers=headers, json=query
                )
                response.raise_for_status()
                parsed = response.json()
                nextToken = parsed.get("next")
                result["items"] += parsed["items"]
                if nextToken is None:
                    break
            return result

        return self._authenticate_request(call)

    def query_devices(self, params):
        """Example params to filter on (all optional)
        {
            name: "model00.001",
            tags: {"location":["sf", "la"]},
        }
        """

        def call(token):
            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            response = requests.post(
                "%s/devices/query" % self._admin_api, headers=headers, json=params
            )
            response.raise_for_status()
            return response.json()

        return self._authenticate_request(call)

    def patch_device(self, device_id, params):
        """Example params
        {
            "desiredConfiguration": 43
        }
        """

        def call(token):
            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            response = requests.patch(
                "%s/devices/%s" % (self._admin_api, device_id),
                headers=headers,
                json=params,
            )
            response.raise_for_status()
            return response.json()

        return self._authenticate_request(call)

    def query_task_summary_formats(self):
        """
        Get all task summary formants
        """
        def call(token):
            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            response = requests.get(
                "%s/task-summary-formats/" % (self._admin_api),
                headers=headers,
            )
            response.raise_for_status()
            return response.json()

        return self._authenticate_request(call)

    def query_task_summaries(self, params):
        """
        Get task summaries
        """

        def call(token):
            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            params["eventTypes"] = ["task-summary"]
            response = requests.post(
                "%s/events/query" % self._admin_api,
                headers=headers,
                json=params,
            )
            response.raise_for_status()
            return response.json()

        return self._authenticate_request(call)

    def upload_task_summary_format(self, task_summary_format):
        """
        Upload a task summary formant
        """

        def call(token):
            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            response = requests.post(
                "%s/task-summary-formats/" % self._admin_api,
                headers=headers,
                json=task_summary_format,
            )
            response.raise_for_status()
            return response.json()

        return self._authenticate_request(call)

    def upload_task_summary(
        self,
        task_summary_format_id,
        report,
        device_id,
        tags=None,
        message=None,
    ):
        """
        Upload a task summary
        """

        def call(token):
            # Try to get start and end time from report
            if "start_time" in report:
                start_time = report["start_time"]
            elif "startTime" in report:
                start_time = report["startTime"]
            else:
                start_time = get_current_isodate()
            if "end_time" in report:
                end_time = report["end_time"]
            elif "startTime" in report:
                end_time = report["endTime"]
            else:
                end_time = get_current_isodate()
            start_time = timestamp_to_datetime_utc(start_time)
            end_time = timestamp_to_datetime_utc(end_time)
            time_now = get_current_isodate()

            # Task summary API doesn't accept None values
            for key in report:
                value = report[key]
                if value is None:
                    raise ValueError("Missing value for key: %s" % key)

            task_summary = {
                "taskSummaryFormatId": task_summary_format_id,
                "report": report,
                "taskId": str(report["id"]),
                "generatedAt": time_now,
                "time": start_time,
                "endTime": end_time,
                "message": message,
                "deviceId": device_id,
                "tags": tags
            }

            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            response = requests.post(
                "%s/task-summaries/" % self._admin_api,
                headers=headers,
                json=task_summary
            )
            response.raise_for_status()
            return response.json()

        return self._authenticate_request(call)

    def query_stream_current_value(self, params):
        """Example query params (all optional):
        {
            start: "2021-01-01T01:00:00.000Z",
            end: "2021-01-01T02:00:00.000Z",
            deviceIds: ["99e8ee37-0a27-4a11-bba2-521facabefa3"],
            names: ["engine_temp"],
            types: ["numeric"],
            tags: {"location":["sf","la"]},
            notNames: ["speed"],
        }
        """

        def call(token):
            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            response = requests.post(
                "%s/stream-current-value" % self._query_api,
                headers=headers,
                json=params,
            )
            response.raise_for_status()
            return response.json()

        return self._authenticate_request(call)

    def upload_file(self, params):
        """
        Upload a file.

        Example params
        {
            path: "/tmp/model.dat"
        }
        """

        file_name = ntpath.basename(params["path"])
        byte_size = os.path.getsize(params["path"])
        if not (byte_size > 0):
            raise ValueError("File is empty")

        def begin_upload(token):
            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            response = requests.post(
                "%s/files/begin-upload" % self._admin_api,
                headers=headers,
                json={"fileName": file_name, "fileSize": byte_size},
            )
            response.raise_for_status()
            return response.json()

        begin_result = self._authenticate_request(begin_upload)
        part_size = begin_result["partSize"]

        etags = []
        part_index = 0
        with open(params["path"], "rb") as file_obj:
            for part_url in begin_result["partUrls"]:
                file_obj.seek(part_index * part_size)
                part_index = part_index + 1
                data = file_obj.read(part_size)
                response = requests.put(part_url, data=data)
                etags.append(response.headers["etag"])

        def complete_upload(token):
            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            response = requests.post(
                "%s/files/complete-upload" % self._admin_api,
                headers=headers,
                json={
                    "fileId": begin_result["fileId"],
                    "uploadId": begin_result["uploadId"],
                    "eTags": etags,
                },
            )
            response.raise_for_status()

        self._authenticate_request(complete_upload)

        return {
            "file_id": begin_result["fileId"],
        }

    def create_command(self, params):
        """
        Create a command.

        Example params
        {
            deviceId: "99e8ee37-0a27-4a11-bba2-521facabefa3"
            command: "return_to_charge_station"
            parameter: {
                "scrubberTime": "2014-11-03T19:38:34.203Z",
                "value": "A-2",
                "files": [{
                    "id": "eb4a823f-58eb-41d6-9b57-c2113261dbbb",
                    "name": "optional_name1"
                }]
            },
        }

        The "value" and "files" keys are optional.
        """

        def call(token):
            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            response = requests.post(
                "%s/commands" % self._admin_api,
                headers=headers,
                json=params,
            )
            response.raise_for_status()
            return response.json()

        return self._authenticate_request(call)

    def demand_device_data(
        self,
        device_id,  # type: str
        start,  # type: datetime.datetime
        end,  # type: datetime.datetime
    ):
        params = {
            "deviceId": device_id,
            "command": FORMANT_REQUEST_ON_DEMAND_DATA_COMMAND_NAME,
            "organizationId": self.get_organization_id(),
            "parameter": {
                "meta": {
                    "start": get_timestamp_str(start),
                    "end": get_timestamp_str(end),
                },
                "scrubberTime": get_current_isodate(),
            },
        }
        return self.create_command(params)

    def query_commands(self, params):
        """
        Query undelivered commands by device ID.

        Example params
        {
            deviceId: "99e8ee37-0a27-4a11-bba2-521facabefa3",
        }
        """

        def call(token):
            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            response = requests.post(
                "%s/commands/query" % self._admin_api,
                headers=headers,
                json=params,
            )
            response.raise_for_status()
            return response.json()

        return self._authenticate_request(call)

    def query_count(
        self,
        start,  # type: datetime.datetime
        end,  # type: datetime.date
        type,  # type: str
    ):
        def call(token):
            params = {
                "start": start.isoformat(),
                "end": end.isoformat(),
                "type": type,
                "organizationId": self.get_organization_id(),
            }
            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            response = requests.post(
                "%s/counts/history" % self._query_api,
                headers=headers,
                json=params,
            )
            response.raise_for_status()
            return response.json()

        return self._authenticate_request(call)

    def active_devices(
        self,
        start,  # type: datetime.datetime
        end,  # type: datetime.datetime
    ):
        def call(token):
            params = {
                "start": start.isoformat(),
                "end": end.isoformat(),
                "type": "default",
                "organizationId": self.get_organization_id(),
            }
            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            response = requests.post(
                "%s/counts/active-devices" % self._query_api,
                headers=headers,
                json=params,
            )
            response.raise_for_status()
            return response.json()

        return self._authenticate_request(call)

    def create_intervention_response(self, params):
        """
        Example intervention response params:
        {
            "interventionId": "518e24fc-64ef-47bb-be5e-036a97aeafaa",
            "interventionType": "teleop",
            "data": {
                "state": "success",
                "notes": "looks good!"
            }
        }
        """

        def call(token):
            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            response = requests.post(
                "%s/intervention-responses" % self._admin_api,
                headers=headers,
                json=params,
            )
            return response.json()

        return self._authenticate_request(call)

    def create_intervention_request(self, params):
        """
        Example intervention request params:
        {
            "message": "A teleop for a customer is requested",
            "interventionType": "teleop",
            "time": "2022-02-17T11:41:33.389-08:00",
            "deviceId": "b306de84-33ca-4917-9218-f686730e24e0",
            "tags": {},
            "data": {
                "instruction": "Look at the users item on the table"
            }
        }
        """

        def call(token):
            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            response = requests.post(
                "%s/intervention-requests" % self._admin_api,
                headers=headers,
                json=params,
            )
            return response.json()

        return self._authenticate_request(call)

    def create_adapter(self, params):
        """
        Example create an adapter params:
        {
            "execCommand": "./start.sh",
            "path": "/tmp/model.dat"
            "name": "adapters_name"
        }
        """

        def call(token):
            file_id = self.upload_file(params={"path": params["path"]})
            params["fileId"] = file_id["file_id"]
            del params["path"]
            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            response = requests.post(
                "%s/adapters" % self._admin_api,
                headers=headers,
                json=params,
            )
            return response.json()

        return self._authenticate_request(call)

    def get_device(self, device_id):
        def call(token):
            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            response = requests.get(
                "%s/devices/%s" % (self._admin_api, device_id), headers=headers
            )
            response.raise_for_status()
            return response.json()

        return self._authenticate_request(call)

    def get_device_configuration(self, device_id, desired_configuration_version):
        def call(token):
            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            response = requests.get(
                "%s/devices/%s/configurations/%s"
                % (self._admin_api, device_id, desired_configuration_version),
                headers=headers,
            )
            response.raise_for_status()
            return response.json()

        return self._authenticate_request(call)

    def post_device_configuration(self, device_id, params):
        """
        Example post configuration params:
        {
            "document": {
                adapter: [{
                    id: "84f98678-5f18-478d-aed8-631d9ea043a9",
                    name: "ROS-diagnostics",
                    "execCommand": "./start.sh"
                    }],
                tags: {},
                telemetry: {
                    streams: []
                }

            },

        }
        """

        def call(token):
            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            response = requests.post(
                "%s/devices/%s/configurations" % (self._admin_api, device_id),
                headers=headers,
                json=params,
            )
            return response.json()

        return self._authenticate_request(call)

    def get_views(self):
        def call(token):
            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            response = requests.get(
                "%s/views/" % self._admin_api,
                headers=headers,
            )
            return response.json()

        return self._authenticate_request(call)

    def get_view(self, view_id):
        def call(token):
            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            response = requests.get(
                "%s/views/%s" % (self._admin_api, view_id),
                headers=headers,
            )
            return response.json()

        return self._authenticate_request(call)

    def patch_view(self, view_id, params):
        def call(token):
            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            response = requests.patch(
                "%s/views/%s" % (self._admin_api, view_id),
                headers=headers,
                json=params,
            )
            return response.json()

        return self._authenticate_request(call)

    def get_annotation_templates(self):
        def call(token):
            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            response = requests.get(
                "%s/annotation-templates/" % self._admin_api,
                headers=headers,
            )
            return response.json()

        return self._authenticate_request(call)

    def post_annotation(self, params):
        def call(token):
            self._add_user_id_to_params(params)
            self._add_organization_id_to_params(params)
            params_stripped = _strip_none_values(params)
            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            response = requests.post(
                "%s/annotations/" % (self._admin_api),
                headers=headers,
                json=params_stripped,
            )
            return response.json()

        return self._authenticate_request(call)

    def query_events(self, params):
        def call(token):
            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            result = {"items": []}
            current_offset = 0
            while True:
                params["offset"] = current_offset
                response = requests.post(
                    "%s/events/query" % (self._admin_api),
                    headers=headers,
                    json=params,
                )
                response.raise_for_status()
                parsed = response.json()
                result["items"] += parsed["items"]
                current_offset += len(parsed["items"])
                if len(parsed["items"]) != 10000:
                    break
            return result

        return self._authenticate_request(call)

    def get_streams(self):
        def call(token):
            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            response = requests.get(
                "%s/streams/" % self._admin_api,
                headers=headers,
            )
            return response.json()

        return self._authenticate_request(call)

    def delete_stream(self, stream_id):
        def call(token):
            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            response = requests.delete(
                "%s/streams/%s" % (self._admin_api, stream_id), headers=headers
            )
            response.raise_for_status()

        return self._authenticate_request(call)

    def query_annotations(
        self,
        start_time=None,  # type: Optional[datetime.datetime]
        device_ids=None,  # type: Optional[List[str]]
        tags=None,  # type: Optional[Dict[str,List[str]]]
        annotation_template_ids=None,  # type: Optional[List[str]]
        message=None,  # type:str
        keyword=None,  # type: str
        end_time=None,  # type: Optional[datetime.datetime]
        params=None,  # type: Dict
    ):
        params = params if params is not None else {}

        params["tags"] = tags
        if start_time is not None:
            params["start"] = start_time.isoformat()
            if end_time is None:
                end_time = datetime.datetime.now(tz=tzutc())
        if end_time is not None:
            params["end"] = end_time.isoformat()
        params["deviceIds"] = device_ids
        params["annotationTemplateIds"] = annotation_template_ids
        params["message"] = message
        params["keyword"] = keyword
        params["eventTypes"] = ["annotation"]

        params_stripped = _strip_none_values(params)
        return self.query_events(params_stripped)

    def _add_user_id_to_params(self, params):
        params["userId"] = self.get_user_id()

    def _add_organization_id_to_params(self, params):
        params["organizationId"] = self.get_organization_id()

    def _authenticate(self):
        payload = {
            "email": self._email,
            "password": self._password,
            "tokenExpirationSeconds": 3600,
        }
        response = requests.post(
            "%s/auth/login" % self._admin_api,
            headers=self._headers,
            json=payload,
        )
        response.raise_for_status()
        result = response.json()
        if "authentication" not in result:
            raise ValueError("Authentication failed")
        self._token_expiry = int(time.time()) + 3530
        self._token = result["authentication"]["accessToken"]
        self._organization_id = result["authentication"]["organizationId"]
        self._user_id = result["authentication"]["userId"]
        self._headers["Org-ID"] = self._organization_id
        self._headers["User-ID"] = self._user_id

    def _authenticate_request(self, call):
        if self._token is None or self._token_expiry < int(time.time()):
            self._authenticate()
        try:
            return call(self._token)
        except requests.exceptions.HTTPError as error:
            if error.response.status_code == 401:
                self._authenticate()
                try:
                    return call(self._token)
                except requests.exceptions.HTTPError as error:
                    sys.stderr.write("%s\n" % error.response.text)
                    raise error
            else:
                sys.stderr.write("%s\n" % error.response.text)
                raise error

    def create_device(
        self,
        device_name,  # type: str
        publicKey="",  # type: str
        tags=None,  # type: Optional[Dict[str,List[str]]]
        params=None,  # type: Dict
    ):
        params = params if params is not None else {}
        params["tags"] = tags
        params["name"] = device_name
        params["publicKey"] = publicKey
        params["enabled"] = True
        params_stripped = _strip_none_values(params)

        def call(token):
            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            response = requests.post(
                "%s/devices" % (self._admin_api),
                headers=headers,
                json=params_stripped,
            )
            response.raise_for_status()
            return response.json()

        return self._authenticate_request(call)

    def generate_provisioning_token(
        self,
        id,  # type: str
        params=None,  # type: Dict
    ):
        params = params if params is not None else {}
        params["id"] = id

        def call(token):
            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            response = requests.post(
                "%s/devices/%s/provisioning-token" % (self._admin_api, id),
                headers=headers,
                json=params,
            )
            response.raise_for_status()
            return response.json()

        return self._authenticate_request(call)

    def provision_device(
        self,
        provisioningToken,  # type: str
        publicKey,  # type: str
        params=None,  # type: Dict
    ):
        params = params if params is not None else {}
        params["provisioningToken"] = provisioningToken
        params["publicKey"] = publicKey
        params_stripped = _strip_none_values(params)

        def call(token):
            headers = self._headers.copy()
            headers["Authorization"] = "Bearer %s" % token
            response = requests.post(
                "%s/devices/provision" % (self._admin_api),
                headers=headers,
                json=params_stripped,
            )
            response.raise_for_status()
            return response.json()

        return self._authenticate_request(call)


def _strip_none_values(dict):
    return {k: v for k, v in dict.items() if v is not None}



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.label import Label


T = TypeVar("T", bound="TagTemplate")


@attr.s(auto_attribs=True)
class TagTemplate:
    """
    Attributes:
        tag_key (Any):
        organization_id (Union[Unset, str]):
        is_group (Union[Unset, bool]):
        is_telemetry_filter (Union[Unset, bool]):
        is_event_filter (Union[Unset, bool]):
        enabled (Union[Unset, bool]):
        description (Union[Unset, None, str]):
        label_set_id (Union[Unset, None, str]):
        label_enum (Union[Unset, None, List['Label']]):
        id (Union[Unset, str]):
        created_at (Union[Unset, datetime.datetime]):
        updated_at (Union[Unset, datetime.datetime]):
    """

    tag_key: Any
    organization_id: Union[Unset, str] = UNSET
    is_group: Union[Unset, bool] = UNSET
    is_telemetry_filter: Union[Unset, bool] = UNSET
    is_event_filter: Union[Unset, bool] = UNSET
    enabled: Union[Unset, bool] = UNSET
    description: Union[Unset, None, str] = UNSET
    label_set_id: Union[Unset, None, str] = UNSET
    label_enum: Union[Unset, None, List["Label"]] = UNSET
    id: Union[Unset, str] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        tag_key = self.tag_key
        organization_id = self.organization_id
        is_group = self.is_group
        is_telemetry_filter = self.is_telemetry_filter
        is_event_filter = self.is_event_filter
        enabled = self.enabled
        description = self.description
        label_set_id = self.label_set_id
        label_enum: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.label_enum, Unset):
            if self.label_enum is None:
                label_enum = None
            else:
                label_enum = []
                for label_enum_item_data in self.label_enum:
                    label_enum_item = label_enum_item_data.to_dict()

                    label_enum.append(label_enum_item)

        id = self.id
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "tagKey": tag_key,
            }
        )
        if organization_id is not UNSET:
            field_dict["organizationId"] = organization_id
        if is_group is not UNSET:
            field_dict["isGroup"] = is_group
        if is_telemetry_filter is not UNSET:
            field_dict["isTelemetryFilter"] = is_telemetry_filter
        if is_event_filter is not UNSET:
            field_dict["isEventFilter"] = is_event_filter
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if description is not UNSET:
            field_dict["description"] = description
        if label_set_id is not UNSET:
            field_dict["labelSetId"] = label_set_id
        if label_enum is not UNSET:
            field_dict["labelEnum"] = label_enum
        if id is not UNSET:
            field_dict["id"] = id
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if updated_at is not UNSET:
            field_dict["updatedAt"] = updated_at

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.label import Label

        d = src_dict.copy()
        tag_key = d.pop("tagKey")

        organization_id = d.pop("organizationId", UNSET)

        is_group = d.pop("isGroup", UNSET)

        is_telemetry_filter = d.pop("isTelemetryFilter", UNSET)

        is_event_filter = d.pop("isEventFilter", UNSET)

        enabled = d.pop("enabled", UNSET)

        description = d.pop("description", UNSET)

        label_set_id = d.pop("labelSetId", UNSET)

        label_enum = []
        _label_enum = d.pop("labelEnum", UNSET)
        for label_enum_item_data in _label_enum or []:
            label_enum_item = Label.from_dict(label_enum_item_data)

            label_enum.append(label_enum_item)

        id = d.pop("id", UNSET)

        _created_at = d.pop("createdAt", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _updated_at = d.pop("updatedAt", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        tag_template = cls(
            tag_key=tag_key,
            organization_id=organization_id,
            is_group=is_group,
            is_telemetry_filter=is_telemetry_filter,
            is_event_filter=is_event_filter,
            enabled=enabled,
            description=description,
            label_set_id=label_set_id,
            label_enum=label_enum,
            id=id,
            created_at=created_at,
            updated_at=updated_at,
        )

        tag_template.additional_properties = d
        return tag_template

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties




from formant.sdk.cloud.v2.formant_admin_api_client.models import CommandParameterMeta, CommandResponseTags, ScopeFilterTypesItem, CommandParameter, CommandDeliveryEventMetadata, CommandRequest, CommandDeliveryEvent, Filter, CommandTemplate, CommandTemplateParameterMeta, CommandResponseStreamType, CommandDeliveryEventType, ScopeFilter, CommandTemplateTags, CommandQuery, CommandProgress, CommandListResponse, CommandStreamType, CommandDeliveryEventStreamType, CommandResponse, CommandTemplateListResponse, FileInfo, CommandRequestListResponse, CommandDeliveryEventTags, Command, CommandTags
from formant.sdk.cloud.v2.formant_admin_api_client.api.command import command_controller_post, command_controller_query
from formant.sdk.cloud.v2.formant_admin_api_client.types import *



from typing import Dict, List, Optional


class AnnotationField:
    def __init__(
        self, annotation_field  # type: Dict
    ):
        self.type = annotation_field["type"]  # type: str
        self.key = annotation_field["key"]  # type: str
        self.name = annotation_field["name"]  # type: str
        self.description = annotation_field.get(
            "description", None
        )  # type: Optional[str]
        self.required = annotation_field["required"]  # type: bool
        parameters = annotation_field.get("parameters", {})

        # The below value is set if type == tag
        self.tag_choices = parameters.get("choices", None)  # type: Optional[List[str]]

        # The below values are set if type == sheet
        self.url = parameters.get("url", None)  # type: Optional[str]
        self.spreadsheet_id = parameters.get(
            "spreadsheetId", None
        )  # type: Optional[str]
        self.range = parameters.get("range", None)  # type: Optional[str]
